@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,30 +14,24 @@
 # limitations under the License.
 
 from __future__ import with_statement
-import errno
+
 import os
 import time
 import traceback
-from datetime import datetime
 from urllib import unquote
-from swift.common.utils import get_logger
+from xml.sax import saxutils
 
-import sqlite3
 from webob import Request, Response
-from webob.exc import HTTPAccepted, HTTPBadRequest, HTTPConflict, \
+from webob.exc import HTTPAccepted, HTTPBadRequest, \
     HTTPCreated, HTTPForbidden, HTTPInternalServerError, \
     HTTPMethodNotAllowed, HTTPNoContent, HTTPNotFound, HTTPPreconditionFailed
 import simplejson
-from xml.sax import saxutils
 
-from swift.common import ACCOUNT_LISTING_LIMIT
 from swift.common.db import AccountBroker
-from swift.common.exceptions import MessageTimeout
-from swift.common.utils import get_param, split_path, storage_directory, \
-    hash_path
-from swift.common.constraints import check_mount, check_float, \
-    check_xml_encodable
-from swift.common.healthcheck import healthcheck
+from swift.common.utils import get_logger, get_param, hash_path, \
+    normalize_timestamp, split_path, storage_directory
+from swift.common.constraints import ACCOUNT_LISTING_LIMIT, \
+    check_mount, check_float, check_utf8
 from swift.common.db_replicator import ReplicatorRpc
 
 
@@ -46,10 +40,9 @@ DATADIR = 'accounts'
 
 class AccountController(object):
     """WSGI controller for the account server."""
-    log_name = 'account'
 
     def __init__(self, conf):
-        self.logger = get_logger(conf, self.log_name)
+        self.logger = get_logger(conf, log_route='account-server')
         self.root = conf.get('devices', '/srv/node')
         self.mount_check = conf.get('mount_check', 'true').lower() in \
                               ('true', 't', '1', 'on', 'yes', 'y')
@@ -83,7 +76,12 @@ class AccountController(object):
 
     def PUT(self, req):
         """Handle HTTP PUT request."""
-        drive, part, account, container = split_path(unquote(req.path), 3, 4)
+        try:
+            drive, part, account, container = split_path(unquote(req.path),
+                                                         3, 4)
+        except ValueError, err:
+            return HTTPBadRequest(body=str(err), content_type='text/plain',
+                                  request=req)
         if self.mount_check and not check_mount(self.root, drive):
             return Response(status='507 %s is not mounted' % drive)
         broker = self._get_account_broker(drive, part, account)
@@ -103,25 +101,39 @@ class AccountController(object):
             else:
                 return HTTPCreated(request=req)
         else:   # put account
+            timestamp = normalize_timestamp(req.headers['x-timestamp'])
             if not os.path.exists(broker.db_file):
-                broker.initialize(req.headers['x-timestamp'])
-                return HTTPCreated(request=req)
+                broker.initialize(timestamp)
+                created = True
             elif broker.is_status_deleted():
                 return HTTPForbidden(request=req, body='Recently deleted')
             else:
-                broker.update_put_timestamp(req.headers['x-timestamp'])
+                created = broker.is_deleted()
+                broker.update_put_timestamp(timestamp)
+                if broker.is_deleted():
+                    return HTTPConflict(request=req)
+            metadata = {}
+            metadata.update((key, (value, timestamp))
+                for key, value in req.headers.iteritems()
+                if key.lower().startswith('x-account-meta-'))
+            if metadata:
+                broker.update_metadata(metadata)
+            if created:
+                return HTTPCreated(request=req)
+            else:
                 return HTTPAccepted(request=req)
 
     def HEAD(self, req):
         """Handle HTTP HEAD request."""
-        # TODO: Refactor: The account server used to provide a 'account and
+        # TODO(refactor): The account server used to provide a 'account and
         # container existence check all-in-one' call by doing a HEAD with a
         # container path. However, container existence is now checked with the
         # container servers directly so this is no longer needed. We should
         # refactor out the container existence check here and retest
         # everything.
         try:
-            drive, part, account, container = split_path(unquote(req.path), 3, 4)
+            drive, part, account, container = split_path(unquote(req.path),
+                                                         3, 4)
         except ValueError, err:
             return HTTPBadRequest(body=str(err), content_type='text/plain',
                                                     request=req)
@@ -139,12 +151,14 @@ class AccountController(object):
             'X-Account-Object-Count': info['object_count'],
             'X-Account-Bytes-Used': info['bytes_used'],
             'X-Timestamp': info['created_at'],
-            'X-PUT-Timestamp': info['put_timestamp'],
-        }
+            'X-PUT-Timestamp': info['put_timestamp']}
         if container:
             container_ts = broker.get_container_timestamp(container)
             if container_ts is not None:
                 headers['X-Container-Timestamp'] = container_ts
+        headers.update((key, value)
+            for key, (value, timestamp) in broker.metadata.iteritems()
+            if value != '')
         return HTTPNoContent(request=req, headers=headers)
 
     def GET(self, req):
@@ -167,8 +181,10 @@ class AccountController(object):
             'X-Account-Object-Count': info['object_count'],
             'X-Account-Bytes-Used': info['bytes_used'],
             'X-Timestamp': info['created_at'],
-            'X-PUT-Timestamp': info['put_timestamp']
-        }
+            'X-PUT-Timestamp': info['put_timestamp']}
+        resp_headers.update((key, value)
+            for key, (value, timestamp) in broker.metadata.iteritems()
+            if value != '')
         try:
             prefix = get_param(req, 'prefix')
             delimiter = get_param(req, 'delimiter')
@@ -183,35 +199,34 @@ class AccountController(object):
                     return  HTTPPreconditionFailed(request=req,
                         body='Maximum limit is %d' % ACCOUNT_LISTING_LIMIT)
             marker = get_param(req, 'marker', '')
+            end_marker = get_param(req, 'end_marker')
             query_format = get_param(req, 'format')
         except UnicodeDecodeError, err:
             return HTTPBadRequest(body='parameters not utf8',
                                   content_type='text/plain', request=req)
-        header_format = req.accept.first_match(['text/plain',
-                                                'application/json',
-                                                'application/xml'])
-        format = query_format if query_format else header_format
-        if format.startswith('application/'):
-            format = format[12:]
-        account_list = broker.list_containers_iter(limit, marker, prefix,
-                                                  delimiter)
-        if format == 'json':
-            out_content_type = 'application/json'
+        if query_format:
+            req.accept = 'application/%s' % query_format.lower()
+        out_content_type = req.accept.best_match(
+                                ['text/plain', 'application/json',
+                                 'application/xml', 'text/xml'],
+                                default_match='text/plain')
+        account_list = broker.list_containers_iter(limit, marker, end_marker,
+                                                   prefix, delimiter)
+        if out_content_type == 'application/json':
             json_pattern = ['"name":%s', '"count":%s', '"bytes":%s']
             json_pattern = '{' + ','.join(json_pattern) + '}'
             json_out = []
             for (name, object_count, bytes_used, is_subdir) in account_list:
                 name = simplejson.dumps(name)
                 if is_subdir:
-                    json_out.append('{"subdir":%s}'% name)
+                    json_out.append('{"subdir":%s}' % name)
                 else:
                     json_out.append(json_pattern %
                         (name, object_count, bytes_used))
             account_list = '[' + ','.join(json_out) + ']'
-        elif format == 'xml':
-            out_content_type = 'application/xml'
+        elif out_content_type.endswith('/xml'):
             output_list = ['<?xml version="1.0" encoding="UTF-8"?>',
-                           '<account name="%s">'%account]
+                           '<account name="%s">' % account]
             for (name, object_count, bytes_used, is_subdir) in account_list:
                 name = saxutils.escape(name)
                 if is_subdir:
@@ -226,16 +241,15 @@ class AccountController(object):
         else:
             if not account_list:
                 return HTTPNoContent(request=req, headers=resp_headers)
-            out_content_type = 'text/plain'
             account_list = '\n'.join(r[0] for r in account_list) + '\n'
         ret = Response(body=account_list, request=req, headers=resp_headers)
         ret.content_type = out_content_type
         ret.charset = 'utf8'
         return ret
 
-    def POST(self, req):
+    def REPLICATE(self, req):
         """
-        Handle HTTP POST request.
+        Handle HTTP REPLICATE request.
         Handler for RPC calls for account replication.
         """
         try:
@@ -247,19 +261,43 @@ class AccountController(object):
         if self.mount_check and not check_mount(self.root, drive):
             return Response(status='507 %s is not mounted' % drive)
         try:
-            args = simplejson.load(req.body_file)
+            args = simplejson.load(req.environ['wsgi.input'])
         except ValueError, err:
             return HTTPBadRequest(body=str(err), content_type='text/plain')
         ret = self.replicator_rpc.dispatch(post_args, args)
         ret.request = req
         return ret
 
+    def POST(self, req):
+        """Handle HTTP POST request."""
+        try:
+            drive, part, account = split_path(unquote(req.path), 3)
+        except ValueError, err:
+            return HTTPBadRequest(body=str(err), content_type='text/plain',
+                                  request=req)
+        if 'x-timestamp' not in req.headers or \
+                not check_float(req.headers['x-timestamp']):
+            return HTTPBadRequest(body='Missing or bad timestamp',
+                request=req, content_type='text/plain')
+        if self.mount_check and not check_mount(self.root, drive):
+            return Response(status='507 %s is not mounted' % drive)
+        broker = self._get_account_broker(drive, part, account)
+        if broker.is_deleted():
+            return HTTPNotFound(request=req)
+        timestamp = normalize_timestamp(req.headers['x-timestamp'])
+        metadata = {}
+        metadata.update((key, (value, timestamp))
+            for key, value in req.headers.iteritems()
+            if key.lower().startswith('x-account-meta-'))
+        if metadata:
+            broker.update_metadata(metadata)
+        return HTTPNoContent(request=req)
+
     def __call__(self, env, start_response):
         start_time = time.time()
         req = Request(env)
-        if req.path_info == '/healthcheck':
-            return healthcheck(req)(env, start_response)
-        elif not check_xml_encodable(req.path_info):
+        self.logger.txn_id = req.headers.get('x-cf-trans-id', None)
+        if not check_utf8(req.path_info):
             res = HTTPPreconditionFailed(body='Invalid UTF8')
         else:
             try:
@@ -267,11 +305,9 @@ class AccountController(object):
                     res = getattr(self, req.method)(req)
                 else:
                     res = HTTPMethodNotAllowed()
-            except:
-                self.logger.exception('ERROR __call__ error with %s %s '
-                    'transaction %s' % (env.get('REQUEST_METHOD', '-'),
-                    env.get('PATH_INFO', '-'), env.get('HTTP_X_CF_TRANS_ID',
-                    '-')))
+            except Exception:
+                self.logger.exception(_('ERROR __call__ error with %(method)s'
+                    ' %(path)s '), {'method': req.method, 'path': req.path})
                 res = HTTPInternalServerError(body=traceback.format_exc())
         trans_time = '%.4f' % (time.time() - start_time)
         additional_info = ''
@@ -287,9 +323,15 @@ class AccountController(object):
             req.referer or '-', req.user_agent or '-',
             trans_time,
             additional_info)
-        if req.method.upper() == 'POST':
+        if req.method.upper() == 'REPLICATE':
             self.logger.debug(log_message)
         else:
             self.logger.info(log_message)
         return res(env, start_response)
 
+
+def app_factory(global_conf, **local_conf):
+    """paste.deploy app factory for creating WSGI account server apps"""
+    conf = global_conf.copy()
+    conf.update(local_conf)
+    return AccountController(conf)
