@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -30,8 +30,8 @@ from urllib import quote
 import logging
 import time
 
-from eventlet.green.httplib import HTTPConnection, HTTPResponse, _UNKNOWN, \
-            CONTINUE, HTTPMessage
+from eventlet.green.httplib import CONTINUE, HTTPConnection, HTTPMessage, \
+    HTTPResponse, HTTPSConnection, _UNKNOWN
 
 
 class BufferedHTTPResponse(HTTPResponse):
@@ -82,15 +82,9 @@ class BufferedHTTPConnection(HTTPConnection):
     def putrequest(self, method, url, skip_host=0, skip_accept_encoding=0):
         self._method = method
         self._path = url
-        self._txn_id = '-'
         return HTTPConnection.putrequest(self, method, url, skip_host,
                                          skip_accept_encoding)
 
-    def putheader(self, header, value):
-        if header.lower() == 'x-cf-trans-id':
-            self._txn_id = value
-        return HTTPConnection.putheader(self, header, value)
-
     def getexpect(self):
         response = BufferedHTTPResponse(self.sock, strict=self.strict,
                                        method=self._method)
@@ -99,17 +93,19 @@ class BufferedHTTPConnection(HTTPConnection):
 
     def getresponse(self):
         response = HTTPConnection.getresponse(self)
-        logging.debug("HTTP PERF: %.5f seconds to %s %s:%s %s (%s)" %
-                (time.time() - self._connected_time, self._method, self.host,
-                 self.port, self._path, self._txn_id))
+        logging.debug(_("HTTP PERF: %(time).5f seconds to %(method)s "
+                        "%(host)s:%(port)s %(path)s)"),
+           {'time': time.time() - self._connected_time, 'method': self._method,
+            'host': self.host, 'port': self.port, 'path': self._path})
         return response
 
 
 def http_connect(ipaddr, port, device, partition, method, path,
-                 headers=None, query_string=None):
+                 headers=None, query_string=None, ssl=False):
     """
-    Helper function to create a HTTPConnection object that is buffered
-    for backend Swift services.
+    Helper function to create an HTTPConnection object. If ssl is set True,
+    HTTPSConnection will be used. However, if ssl=False, BufferedHTTPConnection
+    will be used, which is buffered for backend Swift services.
 
     :param ipaddr: IPv4 address to connect to
     :param port: port to connect to
@@ -119,9 +115,15 @@ def http_connect(ipaddr, port, device, partition, method, path,
     :param path: request path
     :param headers: dictionary of headers
     :param query_string: request query string
+    :param ssl: set True if SSL should be used (default: False)
     :returns: HTTPConnection object
     """
-    conn = BufferedHTTPConnection('%s:%s' % (ipaddr, port))
+    if not port:
+        port = 443 if ssl else 80
+    if ssl:
+        conn = HTTPSConnection('%s:%s' % (ipaddr, port))
+    else:
+        conn = BufferedHTTPConnection('%s:%s' % (ipaddr, port))
     path = quote('/' + device + '/' + str(partition) + path)
     if query_string:
         path += '?' + query_string
@@ -133,10 +135,13 @@ def http_connect(ipaddr, port, device, partition, method, path,
     conn.endheaders()
     return conn
 
+
 def http_connect_raw(ipaddr, port, method, path, headers=None,
-                     query_string=None):
+                     query_string=None, ssl=False):
     """
-    Helper function to create a HTTPConnection object that is buffered.
+    Helper function to create an HTTPConnection object. If ssl is set True,
+    HTTPSConnection will be used. However, if ssl=False, BufferedHTTPConnection
+    will be used, which is buffered for backend Swift services.
 
     :param ipaddr: IPv4 address to connect to
     :param port: port to connect to
@@ -144,9 +149,15 @@ def http_connect_raw(ipaddr, port, method, path, headers=None,
     :param path: request path
     :param headers: dictionary of headers
     :param query_string: request query string
+    :param ssl: set True if SSL should be used (default: False)
     :returns: HTTPConnection object
     """
-    conn = BufferedHTTPConnection('%s:%s' % (ipaddr, port))
+    if not port:
+        port = 443 if ssl else 80
+    if ssl:
+        conn = HTTPSConnection('%s:%s' % (ipaddr, port))
+    else:
+        conn = BufferedHTTPConnection('%s:%s' % (ipaddr, port))
     if query_string:
         path += '?' + query_string
     conn.path = path
