@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -18,17 +18,30 @@ Cloud Files client library used internally
 """
 import socket
 from cStringIO import StringIO
-from httplib import HTTPConnection, HTTPException, HTTPSConnection
 from re import compile, DOTALL
 from tokenize import generate_tokens, STRING, NAME, OP
 from urllib import quote as _quote, unquote
 from urlparse import urlparse, urlunparse
 
+try:
+    from eventlet.green.httplib import HTTPException, HTTPSConnection
+except ImportError:
+    from httplib import HTTPException, HTTPSConnection
+
 try:
     from eventlet import sleep
-except:
+except ImportError:
     from time import sleep
 
+try:
+    from swift.common.bufferedhttp \
+        import BufferedHTTPConnection as HTTPConnection
+except ImportError:
+    try:
+        from eventlet.green.httplib import HTTPConnection
+    except ImportError:
+        from httplib import HTTPConnection
+
 
 def quote(value, safe='/'):
     """
@@ -62,7 +75,7 @@ except ImportError:
                 res = []
                 consts = {'true': True, 'false': False, 'null': None}
                 string = '(' + comments.sub('', string) + ')'
-                for type, val, _, _, _ in \
+                for type, val, _junk, _junk, _junk in \
                         generate_tokens(StringIO(string).readline):
                     if (type == OP and val not in '[]{}:,()-') or \
                             (type == NAME and val not in consts):
@@ -73,7 +86,7 @@ except ImportError:
                     else:
                         res.append(val)
                 return eval(''.join(res), {}, consts)
-            except:
+            except Exception:
                 raise AttributeError()
 
 
@@ -145,19 +158,26 @@ def http_connection(url):
 
 def get_auth(url, user, key, snet=False):
     """
-    Get authentication credentials
+    Get authentication/authorization credentials.
 
-    :param url: authentication URL
-    :param user: user to auth as
-    :param key: key or passowrd for auth
-    :param snet: use SERVICENET internal network default is False 
-    :returns: tuple of (storage URL, storage token, auth token)
+    The snet parameter is used for Rackspace's ServiceNet internal network
+    implementation. In this function, it simply adds *snet-* to the beginning
+    of the host name for the returned storage URL. With Rackspace Cloud Files,
+    use of this network path causes no bandwidth charges but requires the
+    client to be running on Rackspace's ServiceNet network.
+
+    :param url: authentication/authorization URL
+    :param user: user to authenticate as
+    :param key: key or password for authorization
+    :param snet: use SERVICENET internal network (see above), default is False
+    :returns: tuple of (storage URL, auth token)
     :raises ClientException: HTTP GET request to auth URL failed
     """
     parsed, conn = http_connection(url)
     conn.request('GET', parsed.path, '',
                  {'X-Auth-User': user, 'X-Auth-Key': key})
     resp = conn.getresponse()
+    resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Auth GET failed', http_scheme=parsed.scheme,
                 http_host=conn.host, http_port=conn.port,
@@ -166,7 +186,7 @@ def get_auth(url, user, key, snet=False):
     url = resp.getheader('x-storage-url')
     if snet:
         parsed = list(urlparse(url))
-        # Second item in the list is the netloc 
+        # Second item in the list is the netloc
         parsed[1] = 'snet-' + parsed[1]
         url = urlunparse(parsed)
     return url, resp.getheader('x-storage-token',
@@ -187,18 +207,21 @@ def get_account(url, token, marker=None, limit=None, prefix=None,
                       conn object)
     :param full_listing: if True, return a full listing, else returns a max
                          of 10000 listings
-    :returns: a list of accounts
+    :returns: a tuple of (response headers, a list of containers) The response
+              headers will be a dict and all header names will be lowercase.
     :raises ClientException: HTTP GET request failed
     """
     if not http_conn:
         http_conn = http_connection(url)
     if full_listing:
-        rv = []
-        listing = get_account(url, token, marker, limit, prefix, http_conn)
+        rv = get_account(url, token, marker, limit, prefix, http_conn)
+        listing = rv[1]
         while listing:
-            rv.extend(listing)
             marker = listing[-1]['name']
-            listing = get_account(url, token, marker, limit, prefix, http_conn)
+            listing = \
+                get_account(url, token, marker, limit, prefix, http_conn)[1]
+            if listing:
+                rv[1].extend(listing)
         return rv
     parsed, conn = http_conn
     qs = 'format=json'
@@ -211,6 +234,9 @@ def get_account(url, token, marker=None, limit=None, prefix=None,
     conn.request('GET', '%s?%s' % (parsed.path, qs), '',
                  {'X-Auth-Token': token})
     resp = conn.getresponse()
+    resp_headers = {}
+    for header, value in resp.getheaders():
+        resp_headers[header.lower()] = value
     if resp.status < 200 or resp.status >= 300:
         resp.read()
         raise ClientException('Account GET failed', http_scheme=parsed.scheme,
@@ -219,8 +245,8 @@ def get_account(url, token, marker=None, limit=None, prefix=None,
                 http_reason=resp.reason)
     if resp.status == 204:
         resp.read()
-        return []
-    return json_loads(resp.read())
+        return resp_headers, []
+    return resp_headers, json_loads(resp.read())
 
 
 def head_account(url, token, http_conn=None):
@@ -231,7 +257,8 @@ def head_account(url, token, http_conn=None):
     :param token: auth token
     :param http_conn: HTTP connection object (If None, it will create the
                       conn object)
-    :returns: a tuple of (container count, object count, bytes used)
+    :returns: a dict containing the response's headers (all header names will
+              be lowercase)
     :raises ClientException: HTTP HEAD request failed
     """
     if http_conn:
@@ -240,14 +267,42 @@ def head_account(url, token, http_conn=None):
         parsed, conn = http_connection(url)
     conn.request('HEAD', parsed.path, '', {'X-Auth-Token': token})
     resp = conn.getresponse()
+    resp.read()
     if resp.status < 200 or resp.status >= 300:
         raise ClientException('Account HEAD failed', http_scheme=parsed.scheme,
                 http_host=conn.host, http_port=conn.port,
                 http_path=parsed.path, http_status=resp.status,
                 http_reason=resp.reason)
-    return int(resp.getheader('x-account-container-count', 0)), \
-           int(resp.getheader('x-account-object-count', 0)), \
-           int(resp.getheader('x-account-bytes-used', 0))
+    resp_headers = {}
+    for header, value in resp.getheaders():
+        resp_headers[header.lower()] = value
+    return resp_headers
+
+
+def post_account(url, token, headers, http_conn=None):
+    """
+    Update an account's metadata.
+
+    :param url: storage URL
+    :param token: auth token
+    :param headers: additional headers to include in the request
+    :param http_conn: HTTP connection object (If None, it will create the
+                      conn object)
+    :raises ClientException: HTTP POST request failed
+    """
+    if http_conn:
+        parsed, conn = http_conn
+    else:
+        parsed, conn = http_connection(url)
+    headers['X-Auth-Token'] = token
+    conn.request('POST', parsed.path, '', headers)
+    resp = conn.getresponse()
+    resp.read()
+    if resp.status < 200 or resp.status >= 300:
+        raise ClientException('Account POST failed',
+                http_scheme=parsed.scheme, http_host=conn.host,
+                http_port=conn.port, http_path=path, http_status=resp.status,
+                http_reason=resp.reason)
 
 
 def get_container(url, token, container, marker=None, limit=None,
@@ -267,23 +322,25 @@ def get_container(url, token, container, marker=None, limit=None,
                       conn object)
     :param full_listing: if True, return a full listing, else returns a max
                          of 10000 listings
-    :returns: a list of objects
+    :returns: a tuple of (response headers, a list of objects) The response
+              headers will be a dict and all header names will be lowercase.
     :raises ClientException: HTTP GET request failed
     """
     if not http_conn:
         http_conn = http_connection(url)
     if full_listing:
-        rv = []
-        listing = get_container(url, token, container, marker, limit, prefix,
-                                delimiter, http_conn)
+        rv = get_container(url, token, container, marker, limit, prefix,
+                           delimiter, http_conn)
+        listing = rv[1]
         while listing:
-            rv.extend(listing)
             if not delimiter:
                 marker = listing[-1]['name']
             else:
                 marker = listing[-1].get('name', listing[-1].get('subdir'))
             listing = get_container(url, token, container, marker, limit,
-                                    prefix, delimiter, http_conn)
+                                    prefix, delimiter, http_conn)[1]
+            if listing:
+                rv[1].extend(listing)
         return rv
     parsed, conn = http_conn
     path = '%s/%s' % (parsed.path, quote(container))
@@ -304,10 +361,13 @@ def get_container(url, token, container, marker=None, limit=None,
                 http_scheme=parsed.scheme, http_host=conn.host,
                 http_port=conn.port, http_path=path, http_query=qs,
                 http_status=resp.status, http_reason=resp.reason)
+    resp_headers = {}
+    for header, value in resp.getheaders():
+        resp_headers[header.lower()] = value
     if resp.status == 204:
         resp.read()
-        return []
-    return json_loads(resp.read())
+        return resp_headers, []
+    return resp_headers, json_loads(resp.read())
 
 
 def head_container(url, token, container, http_conn=None):
@@ -319,7 +379,8 @@ def head_container(url, token, container, http_conn=None):
     :param container: container name to get stats for
     :param http_conn: HTTP connection object (If None, it will create the
                       conn object)
-    :returns: a tuple of (object count, bytes used)
+    :returns: a dict containing the response's headers (all header names will
+              be lowercase)
     :raises ClientException: HTTP HEAD request failed
     """
     if http_conn:
@@ -335,17 +396,20 @@ def head_container(url, token, container, http_conn=None):
                 http_scheme=parsed.scheme, http_host=conn.host,
                 http_port=conn.port, http_path=path, http_status=resp.status,
                 http_reason=resp.reason)
-    return int(resp.getheader('x-container-object-count', 0)), \
-           int(resp.getheader('x-container-bytes-used', 0))
+    resp_headers = {}
+    for header, value in resp.getheaders():
+        resp_headers[header.lower()] = value
+    return resp_headers
 
 
-def put_container(url, token, container, http_conn=None):
+def put_container(url, token, container, headers=None, http_conn=None):
     """
     Create a container
 
     :param url: storage URL
     :param token: auth token
     :param container: container name to create
+    :param headers: additional headers to include in the request
     :param http_conn: HTTP connection object (If None, it will create the
                       conn object)
     :raises ClientException: HTTP PUT request failed
@@ -355,7 +419,10 @@ def put_container(url, token, container, http_conn=None):
     else:
         parsed, conn = http_connection(url)
     path = '%s/%s' % (parsed.path, quote(container))
-    conn.request('PUT', path, '', {'X-Auth-Token': token})
+    if not headers:
+        headers = {}
+    headers['X-Auth-Token'] = token
+    conn.request('PUT', path, '', headers)
     resp = conn.getresponse()
     resp.read()
     if resp.status < 200 or resp.status >= 300:
@@ -365,6 +432,34 @@ def put_container(url, token, container, http_conn=None):
                 http_reason=resp.reason)
 
 
+def post_container(url, token, container, headers, http_conn=None):
+    """
+    Update a container's metadata.
+
+    :param url: storage URL
+    :param token: auth token
+    :param container: container name to update
+    :param headers: additional headers to include in the request
+    :param http_conn: HTTP connection object (If None, it will create the
+                      conn object)
+    :raises ClientException: HTTP POST request failed
+    """
+    if http_conn:
+        parsed, conn = http_conn
+    else:
+        parsed, conn = http_connection(url)
+    path = '%s/%s' % (parsed.path, quote(container))
+    headers['X-Auth-Token'] = token
+    conn.request('POST', path, '', headers)
+    resp = conn.getresponse()
+    resp.read()
+    if resp.status < 200 or resp.status >= 300:
+        raise ClientException('Container POST failed',
+                http_scheme=parsed.scheme, http_host=conn.host,
+                http_port=conn.port, http_path=path, http_status=resp.status,
+                http_reason=resp.reason)
+
+
 def delete_container(url, token, container, http_conn=None):
     """
     Delete a container
@@ -402,8 +497,12 @@ def get_object(url, token, container, name, http_conn=None,
     :param name: object name to get
     :param http_conn: HTTP connection object (If None, it will create the
                       conn object)
-    :param resp_chunk_size: if defined, chunk size of data to read
-    :returns: a list of objects
+    :param resp_chunk_size: if defined, chunk size of data to read. NOTE: If
+                            you specify a resp_chunk_size you must fully read
+                            the object's contents before making another
+                            request.
+    :returns: a tuple of (response headers, the object's contents) The response
+              headers will be a dict and all header names will be lowercase.
     :raises ClientException: HTTP GET request failed
     """
     if http_conn:
@@ -418,10 +517,6 @@ def get_object(url, token, container, name, http_conn=None,
         raise ClientException('Object GET failed', http_scheme=parsed.scheme,
                 http_host=conn.host, http_port=conn.port, http_path=path,
                 http_status=resp.status, http_reason=resp.reason)
-    metadata = {}
-    for key, value in resp.getheaders():
-        if key.lower().startswith('x-object-meta-'):
-            metadata[unquote(key[len('x-object-meta-'):])] = unquote(value)
     if resp_chunk_size:
 
         def _object_body():
@@ -432,12 +527,10 @@ def get_object(url, token, container, name, http_conn=None,
         object_body = _object_body()
     else:
         object_body = resp.read()
-    return resp.getheader('content-type'), \
-           int(resp.getheader('content-length', 0)), \
-           resp.getheader('last-modified'), \
-           resp.getheader('etag').strip('"'), \
-           metadata, \
-           object_body
+    resp_headers = {}
+    for header, value in resp.getheaders():
+        resp_headers[header.lower()] = value
+    return resp_headers, object_body
 
 
 def head_object(url, token, container, name, http_conn=None):
@@ -450,8 +543,8 @@ def head_object(url, token, container, name, http_conn=None):
     :param name: object name to get info for
     :param http_conn: HTTP connection object (If None, it will create the
                       conn object)
-    :returns: a tuple of (content type, content length, last modfied, etag,
-              dictionary of metadata)
+    :returns: a dict containing the response's headers (all header names will
+              be lowercase)
     :raises ClientException: HTTP HEAD request failed
     """
     if http_conn:
@@ -466,20 +559,15 @@ def head_object(url, token, container, name, http_conn=None):
         raise ClientException('Object HEAD failed', http_scheme=parsed.scheme,
                 http_host=conn.host, http_port=conn.port, http_path=path,
                 http_status=resp.status, http_reason=resp.reason)
-    metadata = {}
-    for key, value in resp.getheaders():
-        if key.lower().startswith('x-object-meta-'):
-            metadata[unquote(key[len('x-object-meta-'):])] = unquote(value)
-    return resp.getheader('content-type'), \
-           int(resp.getheader('content-length', 0)), \
-           resp.getheader('last-modified'), \
-           resp.getheader('etag').strip('"'), \
-           metadata
+    resp_headers = {}
+    for header, value in resp.getheaders():
+        resp_headers[header.lower()] = value
+    return resp_headers
 
 
-def put_object(url, token, container, name, contents, metadata={},
-               content_length=None, etag=None, chunk_size=65536,
-               content_type=None, http_conn=None):
+def put_object(url, token, container, name, contents, content_length=None,
+               etag=None, chunk_size=65536, content_type=None, headers=None,
+               http_conn=None):
     """
     Put an object
 
@@ -487,12 +575,13 @@ def put_object(url, token, container, name, contents, metadata={},
     :param token: auth token
     :param container: container name that the object is in
     :param name: object name to put
-    :param contents: file like object to read object data from
-    :param metadata: dictionary of object metadata
-    :param content_length: value to send as content-length header
+    :param contents: a string or a file like object to read object data from
+    :param content_length: value to send as content-length header; also limits
+                           the amount read from contents
     :param etag: etag of contents
     :param chunk_size: chunk size of data to write
     :param content_type: value to send as content-type header
+    :param headers: additional headers to include in the request
     :param http_conn: HTTP connection object (If None, it will create the
                       conn object)
     :returns: etag from server response
@@ -503,9 +592,9 @@ def put_object(url, token, container, name, contents, metadata={},
     else:
         parsed, conn = http_connection(url)
     path = '%s/%s/%s' % (parsed.path, quote(container), quote(name))
-    headers = {'X-Auth-Token': token}
-    for key, value in metadata.iteritems():
-        headers['X-Object-Meta-%s' % quote(key)] = quote(value)
+    if not headers:
+        headers = {}
+    headers['X-Auth-Token'] = token
     if etag:
         headers['ETag'] = etag.strip('"')
     if content_length is not None:
@@ -518,18 +607,24 @@ def put_object(url, token, container, name, contents, metadata={},
         conn.putrequest('PUT', path)
         for header, value in headers.iteritems():
             conn.putheader(header, value)
-        if not content_length:
+        if content_length is None:
             conn.putheader('Transfer-Encoding', 'chunked')
-        conn.endheaders()
-        chunk = contents.read(chunk_size)
-        while chunk:
-            if not content_length:
-                conn.send('%x\r\n%s\r\n' % (len(chunk), chunk))
-            else:
-                conn.send(chunk)
+            conn.endheaders()
             chunk = contents.read(chunk_size)
-        if not content_length:
+            while chunk:
+                conn.send('%x\r\n%s\r\n' % (len(chunk), chunk))
+                chunk = contents.read(chunk_size)
             conn.send('0\r\n\r\n')
+        else:
+            conn.endheaders()
+            left = content_length
+            while left > 0:
+                size = chunk_size
+                if size > left:
+                    size = left
+                chunk = contents.read(size)
+                conn.send(chunk)
+                left -= len(chunk)
     else:
         conn.request('PUT', path, contents, headers)
     resp = conn.getresponse()
@@ -541,15 +636,15 @@ def put_object(url, token, container, name, contents, metadata={},
     return resp.getheader('etag').strip('"')
 
 
-def post_object(url, token, container, name, metadata, http_conn=None):
+def post_object(url, token, container, name, headers, http_conn=None):
     """
-    Change object metadata
+    Update object metadata
 
     :param url: storage URL
     :param token: auth token
     :param container: container name that the object is in
-    :param name: object name to change
-    :param metadata: dictionary of object metadata
+    :param name: name of the object to update
+    :param headers: additional headers to include in the request
     :param http_conn: HTTP connection object (If None, it will create the
                       conn object)
     :raises ClientException: HTTP POST request failed
@@ -559,9 +654,7 @@ def post_object(url, token, container, name, metadata, http_conn=None):
     else:
         parsed, conn = http_connection(url)
     path = '%s/%s/%s' % (parsed.path, quote(container), quote(name))
-    headers = {'X-Auth-Token': token}
-    for key, value in metadata.iteritems():
-        headers['X-Object-Meta-%s' % quote(key)] = quote(value)
+    headers['X-Auth-Token'] = token
     conn.request('POST', path, '', headers)
     resp = conn.getresponse()
     resp.read()
@@ -602,7 +695,7 @@ class Connection(object):
     """Convenience class to make requests that will also retry the request"""
 
     def __init__(self, authurl, user, key, retries=5, preauthurl=None,
-                 preauthtoken=None, snet=False):
+                 preauthtoken=None, snet=False, starting_backoff=1):
         """
         :param authurl: authenitcation URL
         :param user: user name to authenticate as
@@ -611,7 +704,7 @@ class Connection(object):
         :param preauthurl: storage URL (if you have already authenticated)
         :param preauthtoken: authentication token (if you have already
                              authenticated)
-        :param snet: use SERVICENET internal network default is False 
+        :param snet: use SERVICENET internal network default is False
         """
         self.authurl = authurl
         self.user = user
@@ -622,21 +715,26 @@ class Connection(object):
         self.token = preauthtoken
         self.attempts = 0
         self.snet = snet
+        self.starting_backoff = starting_backoff
 
-    def _retry(self, func, *args, **kwargs):
-        kwargs['http_conn'] = self.http_conn
+    def get_auth(self):
+        return get_auth(self.authurl, self.user, self.key, snet=self.snet)
+
+    def http_connection(self):
+        return http_connection(self.url)
+
+    def _retry(self, reset_func, func, *args, **kwargs):
         self.attempts = 0
-        backoff = 1
+        backoff = self.starting_backoff
         while self.attempts <= self.retries:
             self.attempts += 1
             try:
                 if not self.url or not self.token:
-                    self.url, self.token = \
-                        get_auth(self.authurl, self.user, self.key, snet=self.snet)
+                    self.url, self.token = self.get_auth()
                     self.http_conn = None
                 if not self.http_conn:
-                    self.http_conn = http_connection(self.url)
-                    kwargs['http_conn'] = self.http_conn
+                    self.http_conn = self.http_connection()
+                kwargs['http_conn'] = self.http_conn
                 rv = func(self.url, self.token, *args, **kwargs)
                 return rv
             except (socket.error, HTTPException):
@@ -656,63 +754,85 @@ class Connection(object):
                     raise
             sleep(backoff)
             backoff *= 2
+            if reset_func:
+                reset_func(func, *args, **kwargs)
 
     def head_account(self):
-        """Wrapper for head_account"""
-        return self._retry(head_account)
+        """Wrapper for :func:`head_account`"""
+        return self._retry(None, head_account)
 
     def get_account(self, marker=None, limit=None, prefix=None,
                     full_listing=False):
-        """Wrapper for get_account"""
-        # TODO: With full_listing=True this will restart the entire listing
-        # with each retry. Need to make a better version that just retries
-        # where it left off.
-        return self._retry(get_account, marker=marker, limit=limit,
+        """Wrapper for :func:`get_account`"""
+        # TODO(unknown): With full_listing=True this will restart the entire
+        # listing with each retry. Need to make a better version that just
+        # retries where it left off.
+        return self._retry(None, get_account, marker=marker, limit=limit,
                            prefix=prefix, full_listing=full_listing)
 
+    def post_account(self, headers):
+        """Wrapper for :func:`post_account`"""
+        return self._retry(None, post_account, headers)
+
     def head_container(self, container):
-        """Wrapper for head_container"""
-        return self._retry(head_container, container)
+        """Wrapper for :func:`head_container`"""
+        return self._retry(None, head_container, container)
 
     def get_container(self, container, marker=None, limit=None, prefix=None,
                       delimiter=None, full_listing=False):
-        """Wrapper for get_container"""
-        # TODO: With full_listing=True this will restart the entire listing
-        # with each retry. Need to make a better version that just retries
-        # where it left off.
-        return self._retry(get_container, container, marker=marker,
+        """Wrapper for :func:`get_container`"""
+        # TODO(unknown): With full_listing=True this will restart the entire
+        # listing with each retry. Need to make a better version that just
+        # retries where it left off.
+        return self._retry(None, get_container, container, marker=marker,
                            limit=limit, prefix=prefix, delimiter=delimiter,
                            full_listing=full_listing)
 
-    def put_container(self, container):
-        """Wrapper for put_container"""
-        return self._retry(put_container, container)
+    def put_container(self, container, headers=None):
+        """Wrapper for :func:`put_container`"""
+        return self._retry(None, put_container, container, headers=headers)
+
+    def post_container(self, container, headers):
+        """Wrapper for :func:`post_container`"""
+        return self._retry(None, post_container, container, headers)
 
     def delete_container(self, container):
-        """Wrapper for delete_container"""
-        return self._retry(delete_container, container)
+        """Wrapper for :func:`delete_container`"""
+        return self._retry(None, delete_container, container)
 
     def head_object(self, container, obj):
-        """Wrapper for head_object"""
-        return self._retry(head_object, container, obj)
+        """Wrapper for :func:`head_object`"""
+        return self._retry(None, head_object, container, obj)
 
     def get_object(self, container, obj, resp_chunk_size=None):
-        """Wrapper for get_object"""
-        return self._retry(get_object, container, obj,
+        """Wrapper for :func:`get_object`"""
+        return self._retry(None, get_object, container, obj,
                            resp_chunk_size=resp_chunk_size)
 
-    def put_object(self, container, obj, contents, metadata={},
-                   content_length=None, etag=None, chunk_size=65536,
-                   content_type=None):
-        """Wrapper for put_object"""
-        return self._retry(put_object, container, obj, contents,
-            metadata=metadata, content_length=content_length, etag=etag,
-            chunk_size=chunk_size, content_type=content_type)
+    def put_object(self, container, obj, contents, content_length=None,
+                   etag=None, chunk_size=65536, content_type=None,
+                   headers=None):
+        """Wrapper for :func:`put_object`"""
 
-    def post_object(self, container, obj, metadata):
-        """Wrapper for post_object"""
-        return self._retry(post_object, container, obj, metadata)
+        def _default_reset(*args, **kwargs):
+            raise ClientException('put_object(%r, %r, ...) failure and no '
+                'ability to reset contents for reupload.' % (container, obj))
+
+        reset_func = _default_reset
+        tell = getattr(contents, 'tell', None)
+        seek = getattr(contents, 'seek', None)
+        if tell and seek:
+            orig_pos = tell()
+            reset_func = lambda *a, **k: seek(orig_pos)
+
+        return self._retry(reset_func, put_object, container, obj, contents,
+            content_length=content_length, etag=etag, chunk_size=chunk_size,
+            content_type=content_type, headers=headers)
+
+    def post_object(self, container, obj, headers):
+        """Wrapper for :func:`post_object`"""
+        return self._retry(None, post_object, container, obj, headers)
 
     def delete_object(self, container, obj):
-        """Wrapper for delete_object"""
-        return self._retry(delete_object, container, obj)
+        """Wrapper for :func:`delete_object`"""
+        return self._retry(None, delete_object, container, obj)
