@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -21,7 +21,7 @@ import math
 import time
 import shutil
 
-from eventlet import GreenPool, sleep, Timeout
+from eventlet import GreenPool, sleep, Timeout, TimeoutError
 from eventlet.green import subprocess
 import simplejson
 from webob import Response
@@ -33,6 +33,7 @@ from swift.common.utils import get_logger, whataremyips, storage_directory, \
 from swift.common import ring
 from swift.common.bufferedhttp import BufferedHTTPConnection
 from swift.common.exceptions import DriveNotMounted, ConnectionTimeout
+from swift.common.daemon import Daemon
 
 
 def quarantine_db(object_file, server_type):
@@ -53,8 +54,9 @@ def quarantine_db(object_file, server_type):
 
 class ReplConnection(BufferedHTTPConnection):
     """
-    Helper to simplify POSTing to a remote server.
+    Helper to simplify REPLICATEing to a remote server.
     """
+
     def __init__(self, node, partition, hash_, logger):
         ""
         self.logger = logger
@@ -62,9 +64,9 @@ class ReplConnection(BufferedHTTPConnection):
         BufferedHTTPConnection.__init__(self, '%(ip)s:%(port)s' % node)
         self.path = '/%s/%s/%s' % (node['device'], partition, hash_)
 
-    def post(self, *args):
+    def replicate(self, *args):
         """
-        Make an HTTP POST request
+        Make an HTTP REPLICATE request
 
         :param args: list of json-encodable objects
 
@@ -72,44 +74,40 @@ class ReplConnection(BufferedHTTPConnection):
         """
         try:
             body = simplejson.dumps(args)
-            self.request('POST', self.path, body,
+            self.request('REPLICATE', self.path, body,
                     {'Content-Type': 'application/json'})
             response = self.getresponse()
             response.data = response.read()
             return response
-        except:
+        except Exception:
             self.logger.exception(
-                'ERROR reading HTTP response from %s' % self.node)
+                _('ERROR reading HTTP response from %s'), self.node)
             return None
 
 
-class Replicator(object):
+class Replicator(Daemon):
     """
     Implements the logic for directing db replication.
     """
 
-    def __init__(self, server_conf, replicator_conf):
-        self.logger = \
-            get_logger(replicator_conf, '%s-replicator' % self.server_type)
-        # log uncaught exceptions
-        sys.excepthook = lambda *exc_info: \
-                self.logger.critical('UNCAUGHT EXCEPTION', exc_info=exc_info)
-        sys.stdout = sys.stderr = LoggerFileObject(self.logger)
-        self.root = server_conf.get('devices', '/srv/node')
-        self.mount_check = server_conf.get('mount_check', 'true').lower() in \
+    def __init__(self, conf):
+        self.conf = conf
+        self.logger = get_logger(conf, log_route='replicator')
+        self.root = conf.get('devices', '/srv/node')
+        self.mount_check = conf.get('mount_check', 'true').lower() in \
                               ('true', 't', '1', 'on', 'yes', 'y')
-        self.port = int(server_conf.get('bind_port', self.default_port))
-        concurrency = int(replicator_conf.get('concurrency', 8))
+        self.port = int(conf.get('bind_port', self.default_port))
+        concurrency = int(conf.get('concurrency', 8))
         self.cpool = GreenPool(size=concurrency)
-        swift_dir = server_conf.get('swift_dir', '/etc/swift')
+        swift_dir = conf.get('swift_dir', '/etc/swift')
         self.ring = ring.Ring(os.path.join(swift_dir, self.ring_file))
-        self.per_diff = int(replicator_conf.get('per_diff', 1000))
-        self.run_pause = int(replicator_conf.get('run_pause', 30))
-        self.vm_test_mode = replicator_conf.get(
+        self.per_diff = int(conf.get('per_diff', 1000))
+        self.run_pause = int(conf.get('run_pause', 30))
+        self.vm_test_mode = conf.get(
             'vm_test_mode', 'no').lower() in ('yes', 'true', 'on', '1')
-        self.node_timeout = int(replicator_conf.get('node_timeout', 10))
-        self.conn_timeout = float(replicator_conf.get('conn_timeout', 0.5))
-        self.reclaim_age = float(replicator_conf.get('reclaim_age', 86400 * 7))
+        self.node_timeout = int(conf.get('node_timeout', 10))
+        self.conn_timeout = float(conf.get('conn_timeout', 0.5))
+        self.reclaim_age = float(conf.get('reclaim_age', 86400 * 7))
         self._zero_stats()
 
     def _zero_stats(self):
@@ -122,12 +120,14 @@ class Replicator(object):
     def _report_stats(self):
         """Report the current stats to the logs."""
         self.logger.info(
-            'Attempted to replicate %d dbs in %.5f seconds (%.5f/s)'
-            % (self.stats['attempted'], time.time() - self.stats['start'],
-               self.stats['attempted'] /
-                        (time.time() - self.stats['start'] + 0.0000001)))
-        self.logger.info('Removed %(remove)d dbs' % self.stats)
-        self.logger.info('%(success)s successes, %(failure)s failures'
+            _('Attempted to replicate %(count)d dbs in %(time).5f seconds '
+              '(%(rate).5f/s)'),
+            {'count': self.stats['attempted'],
+             'time': time.time() - self.stats['start'],
+             'rate': self.stats['attempted'] /
+                        (time.time() - self.stats['start'] + 0.0000001)})
+        self.logger.info(_('Removed %(remove)d dbs') % self.stats)
+        self.logger.info(_('%(success)s successes, %(failure)s failures')
             % self.stats)
         self.logger.info(' '.join(['%s:%s' % item for item in
              self.stats.items() if item[0] in
@@ -152,12 +152,12 @@ class Replicator(object):
         proc = subprocess.Popen(popen_args)
         proc.communicate()
         if proc.returncode != 0:
-            self.logger.error('ERROR rsync failed with %s: %s' %
-                              (proc.returncode, popen_args))
+            self.logger.error(_('ERROR rsync failed with %(code)s: %(args)s'),
+                              {'code': proc.returncode, 'args': popen_args})
         return proc.returncode == 0
 
     def _rsync_db(self, broker, device, http, local_id,
-            post_method='complete_rsync', post_timeout=None):
+            replicate_method='complete_rsync', replicate_timeout=None):
         """
         Sync a whole db using rsync.
 
@@ -165,8 +165,8 @@ class Replicator(object):
         :param device: device to sync to
         :param http: ReplConnection object
         :param local_id: unique ID of the local database replica
-        :param post_method: remote operation to perform after rsync
-        :param post_timeout: timeout to wait in seconds
+        :param replicate_method: remote operation to perform after rsync
+        :param replicate_timeout: timeout to wait in seconds
         """
         if self.vm_test_mode:
             remote_file = '%s::%s%s/%s/tmp/%s' % (device['ip'],
@@ -185,8 +185,8 @@ class Replicator(object):
             with broker.lock():
                 if not self._rsync_file(broker.db_file, remote_file, False):
                     return False
-        with Timeout(post_timeout or self.node_timeout):
-            response = http.post(post_method, local_id)
+        with Timeout(replicate_timeout or self.node_timeout):
+            response = http.replicate(replicate_method, local_id)
         return response and response.status >= 200 and response.status < 300
 
     def _usync_db(self, point, broker, http, remote_id, local_id):
@@ -202,21 +202,22 @@ class Replicator(object):
         :returns: boolean indicating completion and success
         """
         self.stats['diff'] += 1
-        self.logger.debug('Syncing chunks with %s', http.host)
+        self.logger.debug(_('Syncing chunks with %s'), http.host)
         sync_table = broker.get_syncs()
         objects = broker.get_items_since(point, self.per_diff)
         while len(objects):
             with Timeout(self.node_timeout):
-                response = http.post('merge_items', objects, local_id)
+                response = http.replicate('merge_items', objects, local_id)
             if not response or response.status >= 300 or response.status < 200:
                 if response:
-                    self.logger.error('ERROR Bad response %s from %s' %
-                        (response.status, http.host))
+                    self.logger.error(_('ERROR Bad response %(status)s from '
+                        '%(host)s'),
+                        {'status': response.status, 'host': http.host})
                 return False
             point = objects[-1]['ROWID']
             objects = broker.get_items_since(point, self.per_diff)
         with Timeout(self.node_timeout):
-            response = http.post('merge_syncs', sync_table)
+            response = http.replicate('merge_syncs', sync_table)
         if response and response.status >= 200 and response.status < 300:
             broker.merge_syncs([{'remote_id': remote_id,
                     'sync_point': point}], incoming=False)
@@ -265,7 +266,8 @@ class Replicator(object):
         :param broker: DB broker for the DB to be replication
         :param partition: partition on the node to replicate to
         :param info: DB info as a dictionary of {'max_row', 'hash', 'id',
-                     'created_at', 'put_timestamp', 'delete_timestamp'}
+                     'created_at', 'put_timestamp', 'delete_timestamp',
+                     'metadata'}
 
         :returns: True if successful, False otherwise
         """
@@ -273,12 +275,12 @@ class Replicator(object):
             http = self._http_connect(node, partition, broker.db_file)
         if not http:
             self.logger.error(
-                'ERROR Unable to connect to remote server: %s' % node)
+                _('ERROR Unable to connect to remote server: %s'), node)
             return False
         with Timeout(self.node_timeout):
-            response = http.post('sync', info['max_row'], info['hash'],
+            response = http.replicate('sync', info['max_row'], info['hash'],
                 info['id'], info['created_at'], info['put_timestamp'],
-                info['delete_timestamp'])
+                info['delete_timestamp'], info['metadata'])
         if not response:
             return False
         elif response.status == HTTPNotFound.code:  # completely missing, rsync
@@ -296,8 +298,8 @@ class Replicator(object):
             if rinfo['max_row'] / float(info['max_row']) < 0.5:
                 self.stats['remote_merge'] += 1
                 return self._rsync_db(broker, node, http, info['id'],
-                        post_method='rsync_then_merge',
-                        post_timeout=(info['count'] / 2000))
+                        replicate_method='rsync_then_merge',
+                        replicate_timeout=(info['count'] / 2000))
             # else send diffs over to the remote server
             return self._usync_db(max(rinfo['point'], local_sync),
                         broker, http, rinfo['id'], info['id'])
@@ -311,7 +313,7 @@ class Replicator(object):
         :param object_file: DB file name to be replicated
         :param node_id: node id of the node to be replicated to
         """
-        self.logger.debug('Replicating db %s' % object_file)
+        self.logger.debug(_('Replicating db %s'), object_file)
         self.stats['attempted'] += 1
         try:
             broker = self.brokerclass(object_file, pending_timeout=30)
@@ -320,10 +322,10 @@ class Replicator(object):
             info = broker.get_replication_info()
         except Exception, e:
             if 'no such table' in str(e):
-                self.logger.error('Quarantining DB %s' % object_file)
+                self.logger.error(_('Quarantining DB %s'), object_file)
                 quarantine_db(broker.db_file, broker.db_type)
             else:
-                self.logger.exception('ERROR reading db %s' % object_file)
+                self.logger.exception(_('ERROR reading db %s'), object_file)
             self.stats['failure'] += 1
             return
         # The db is considered deleted if the delete_timestamp value is greater
@@ -356,10 +358,10 @@ class Replicator(object):
                 success = self._repl_to_node(node, broker, partition, info)
             except DriveNotMounted:
                 repl_nodes.append(more_nodes.next())
-                self.logger.error('ERROR Remote drive not mounted %s' % node)
-            except:
-                self.logger.exception('ERROR syncing %s with node %s' %
-                        (object_file, node))
+                self.logger.error(_('ERROR Remote drive not mounted %s'), node)
+            except (Exception, TimeoutError):
+                self.logger.exception(_('ERROR syncing %(file)s with node'
+                        ' %(node)s'), {'file': object_file, 'node': node})
             self.stats['success' if success else 'failure'] += 1
             responses.append(success)
         if not shouldbehere and all(responses):
@@ -376,6 +378,7 @@ class Replicator(object):
 
         :param datadirs: a list of paths to walk
         """
+
         def walk_datadir(datadir, node_id):
             partitions = os.listdir(datadir)
             random.shuffle(partitions)
@@ -393,20 +396,20 @@ class Replicator(object):
                 except StopIteration:
                     its.remove(it)
 
-    def replicate_once(self):
+    def run_once(self, *args, **kwargs):
         """Run a replication pass once."""
         self._zero_stats()
         dirs = []
         ips = whataremyips()
         if not ips:
-            self.logger.error('ERROR Failed to get my own IPs?')
+            self.logger.error(_('ERROR Failed to get my own IPs?'))
             return
         for node in self.ring.devs:
             if node and node['ip'] in ips and node['port'] == self.port:
                 if self.mount_check and not os.path.ismount(
                         os.path.join(self.root, node['device'])):
                     self.logger.warn(
-                        'Skipping %(device)s as it is not mounted' % node)
+                        _('Skipping %(device)s as it is not mounted') % node)
                     continue
                 unlink_older_than(
                     os.path.join(self.root, node['device'], 'tmp'),
@@ -414,28 +417,28 @@ class Replicator(object):
                 datadir = os.path.join(self.root, node['device'], self.datadir)
                 if os.path.isdir(datadir):
                     dirs.append((datadir, node['id']))
-        self.logger.info('Beginning replication run')
+        self.logger.info(_('Beginning replication run'))
         for part, object_file, node_id in self.roundrobin_datadirs(dirs):
             self.cpool.spawn_n(
                 self._replicate_object, part, object_file, node_id)
         self.cpool.waitall()
-        self.logger.info('Replication run OVER')
+        self.logger.info(_('Replication run OVER'))
         self._report_stats()
 
-    def replicate_forever(self):
+    def run_forever(self, *args, **kwargs):
         """
         Replicate dbs under the given root in an infinite loop.
         """
         while True:
             try:
-                self.replicate_once()
-            except:
-                self.logger.exception('ERROR trying to replicate')
+                self.run_once()
+            except (Exception, TimeoutError):
+                self.logger.exception(_('ERROR trying to replicate'))
             sleep(self.run_pause)
 
 
 class ReplicatorRpc(object):
-    """Handle Replication RPC calls.  TODO: redbo document please :)"""
+    """Handle Replication RPC calls.  TODO(redbo): document please :)"""
 
     def __init__(self, root, datadir, broker_class, mount_check=True):
         self.root = root
@@ -443,11 +446,11 @@ class ReplicatorRpc(object):
         self.broker_class = broker_class
         self.mount_check = mount_check
 
-    def dispatch(self, post_args, args):
+    def dispatch(self, replicate_args, args):
         if not hasattr(args, 'pop'):
             return HTTPBadRequest(body='Invalid object type')
         op = args.pop(0)
-        drive, partition, hsh = post_args
+        drive, partition, hsh = replicate_args
         if self.mount_check and \
                 not os.path.ismount(os.path.join(self.root, drive)):
             return Response(status='507 %s is not mounted' % drive)
@@ -467,16 +470,18 @@ class ReplicatorRpc(object):
 
     def sync(self, broker, args):
         (remote_sync, hash_, id_, created_at, put_timestamp,
-         delete_timestamp) = args
+         delete_timestamp, metadata) = args
         try:
             info = broker.get_replication_info()
         except Exception, e:
             if 'no such table' in str(e):
-                # TODO find a real logger
-                print "Quarantining DB %s" % broker.db_file
+                # TODO(unknown): find a real logger
+                print _("Quarantining DB %s") % broker.db_file
                 quarantine_db(broker.db_file, broker.db_type)
                 return HTTPNotFound()
             raise
+        if metadata:
+            broker.update_metadata(simplejson.loads(metadata))
         if info['put_timestamp'] != put_timestamp or \
                     info['created_at'] != created_at or \
                     info['delete_timestamp'] != delete_timestamp:
