@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -26,10 +26,37 @@ class MessageTimeout(TimeoutError):
         return '%s: %s' % (TimeoutError.__str__(self), self.msg)
 
 
-class AuditException(Exception): pass
-class AuthException(Exception): pass
-class ChunkReadTimeout(TimeoutError): pass
-class ChunkWriteTimeout(TimeoutError): pass
-class ConnectionTimeout(TimeoutError): pass
-class DriveNotMounted(Exception): pass
-class LockTimeout(MessageTimeout): pass
+class AuditException(Exception):
+    pass
+
+
+class DiskFileError(Exception):
+    pass
+
+
+class DiskFileNotExist(Exception):
+    pass
+
+
+class AuthException(Exception):
+    pass
+
+
+class ChunkReadTimeout(TimeoutError):
+    pass
+
+
+class ChunkWriteTimeout(TimeoutError):
+    pass
+
+
+class ConnectionTimeout(TimeoutError):
+    pass
+
+
+class DriveNotMounted(Exception):
+    pass
+
+
+class LockTimeout(MessageTimeout):
+    pass
