@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,8 +14,7 @@
 # limitations under the License.
 
 from array import array
-from bisect import bisect
-from random import randint
+from random import randint, shuffle
 from time import time
 
 from swift.common.ring import RingData
@@ -70,6 +69,49 @@ class RingBuilder(object):
         self._remove_devs = []
         self._ring = None
 
+    def copy_from(self, builder):
+        if hasattr(builder, 'devs'):
+            self.part_power = builder.part_power
+            self.replicas = builder.replicas
+            self.min_part_hours = builder.min_part_hours
+            self.parts = builder.parts
+            self.devs = builder.devs
+            self.devs_changed = builder.devs_changed
+            self.version = builder.version
+            self._replica2part2dev = builder._replica2part2dev
+            self._last_part_moves_epoch = builder._last_part_moves_epoch
+            self._last_part_moves = builder._last_part_moves
+            self._last_part_gather_start = builder._last_part_gather_start
+            self._remove_devs = builder._remove_devs
+        else:
+            self.part_power = builder['part_power']
+            self.replicas = builder['replicas']
+            self.min_part_hours = builder['min_part_hours']
+            self.parts = builder['parts']
+            self.devs = builder['devs']
+            self.devs_changed = builder['devs_changed']
+            self.version = builder['version']
+            self._replica2part2dev = builder['_replica2part2dev']
+            self._last_part_moves_epoch = builder['_last_part_moves_epoch']
+            self._last_part_moves = builder['_last_part_moves']
+            self._last_part_gather_start = builder['_last_part_gather_start']
+            self._remove_devs = builder['_remove_devs']
+        self._ring = None
+
+    def to_dict(self):
+        return {'part_power': self.part_power,
+                'replicas': self.replicas,
+                'min_part_hours': self.min_part_hours,
+                'parts': self.parts,
+                'devs': self.devs,
+                'devs_changed': self.devs_changed,
+                'version': self.version,
+                '_replica2part2dev': self._replica2part2dev,
+                '_last_part_moves_epoch': self._last_part_moves_epoch,
+                '_last_part_moves': self._last_part_moves,
+                '_last_part_gather_start': self._last_part_gather_start,
+                '_remove_devs': self._remove_devs}
+
     def change_min_part_hours(self, min_part_hours):
         """
         Changes the value used to decide if a given partition can be moved
@@ -101,7 +143,10 @@ class RingBuilder(object):
                     continue
                 devs[dev['id']] = dict((k, v) for k, v in dev.items()
                                        if k not in ('parts', 'parts_wanted'))
-            self._ring = \
+            if not self._replica2part2dev:
+                self._ring = RingData([], devs, 32 - self.part_power)
+            else:
+                self._ring = \
                 RingData([array('H', p2d) for p2d in self._replica2part2dev],
                          devs, 32 - self.part_power)
         return self._ring
@@ -237,7 +282,7 @@ class RingBuilder(object):
                 (sum(d['parts'] for d in self.devs if d is not None),
                  self.parts * self.replicas))
         if stats:
-            dev_usage = array('I', (0 for _ in xrange(len(self.devs))))
+            dev_usage = array('I', (0 for _junk in xrange(len(self.devs))))
         for part in xrange(self.parts):
             zones = {}
             for replica in xrange(self.replicas):
@@ -340,8 +385,9 @@ class RingBuilder(object):
                 '%08x.%04x' % (dev['parts_wanted'], randint(0, 0xffff))
         available_devs = sorted((d for d in self.devs if d is not None),
                                 key=lambda x: x['sort_key'])
-        self._replica2part2dev = [array('H') for _ in xrange(self.replicas)]
-        for _ in xrange(self.parts):
+        self._replica2part2dev = \
+            [array('H') for _junk in xrange(self.replicas)]
+        for _junk in xrange(self.parts):
             other_zones = array('H')
             for replica in xrange(self.replicas):
                 index = len(available_devs) - 1
@@ -363,7 +409,7 @@ class RingBuilder(object):
                         index = mid + 1
                 available_devs.insert(index, dev)
                 other_zones.append(dev['zone'])
-        self._last_part_moves = array('B', (0 for _ in xrange(self.parts)))
+        self._last_part_moves = array('B', (0 for _junk in xrange(self.parts)))
         self._last_part_moves_epoch = int(time())
         for dev in self.devs:
             del dev['sort_key']
@@ -411,6 +457,7 @@ class RingBuilder(object):
                         dev['parts_wanted'] += 1
                         dev['parts'] -= 1
                         reassign_parts.append(part)
+        shuffle(reassign_parts)
         return reassign_parts
 
     def _reassign_parts(self, reassign_parts):
