@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,20 +15,25 @@
 
 import cPickle as pickle
 from gzip import GzipFile
-from hashlib import md5
 from os.path import getmtime
 from struct import unpack_from
 from time import time
-from swift.common.utils import hash_path
+from swift.common.utils import hash_path, validate_configuration
 
 
 class RingData(object):
     """Partitioned consistent hashing ring data (used for serialization)."""
+
     def __init__(self, replica2part2dev_id, devs, part_shift):
         self.devs = devs
         self._replica2part2dev_id = replica2part2dev_id
         self._part_shift = part_shift
 
+    def to_dict(self):
+        return {'devs': self.devs,
+                'replica2part2dev_id': self._replica2part2dev_id,
+                'part_shift': self._part_shift}
+
 
 class Ring(object):
     """
@@ -37,7 +42,10 @@ class Ring(object):
     :param pickle_gz_path: path to ring file
     :param reload_time: time interval in seconds to check for a ring change
     """
+
     def __init__(self, pickle_gz_path, reload_time=15):
+        # can't use the ring unless HASH_PATH_SUFFIX is set
+        validate_configuration()
         self.pickle_gz_path = pickle_gz_path
         self.reload_time = reload_time
         self._reload(force=True)
@@ -46,6 +54,9 @@ class Ring(object):
         self._rtime = time() + self.reload_time
         if force or self.has_changed():
             ring_data = pickle.load(GzipFile(self.pickle_gz_path, 'rb'))
+            if not hasattr(ring_data, 'devs'):
+                ring_data = RingData(ring_data['replica2part2dev_id'],
+                    ring_data['devs'], ring_data['part_shift'])
             self._mtime = getmtime(self.pickle_gz_path)
             self.devs = ring_data.devs
             self.zone2devs = {}
@@ -138,4 +149,12 @@ class Ring(object):
             zones.remove(self.devs[part2dev_id[part]]['zone'])
         while zones:
             zone = zones.pop(part % len(zones))
-            yield self.zone2devs[zone][part % len(self.zone2devs[zone])]
+            weighted_node = None
+            for i in xrange(len(self.zone2devs[zone])):
+                node = self.zone2devs[zone][(part + i) %
+                                            len(self.zone2devs[zone])]
+                if node.get('weight'):
+                    weighted_node = node
+                    break
+            if weighted_node:
+                yield weighted_node
