@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -24,6 +24,7 @@ import mimetools
 
 import eventlet
 from eventlet import greenio, GreenPool, sleep, wsgi, listen
+from paste.deploy import loadapp, appconfig
 
 # Hook to ensure connection resets don't blow up our servers.
 # Remove with next release of Eventlet that has it in the set already.
@@ -33,7 +34,8 @@ wsgi.ACCEPT_ERRNO.add(ECONNRESET)
 from eventlet.green import socket, ssl
 
 from swift.common.utils import get_logger, drop_privileges, \
-    LoggerFileObject, NullLogger
+    validate_configuration, capture_stdio, NullLogger
+
 
 def monkey_patch_mimetools():
     """
@@ -54,36 +56,27 @@ def monkey_patch_mimetools():
 
     mimetools.Message.parsetype = parsetype
 
-# We might be able to pull pieces of this out to test, but right now it seems
-# like more work than it's worth.
-def run_wsgi(app, conf, *args, **kwargs):   # pragma: no cover
-    """
-    Loads common settings from conf, then instantiates app and runs
-    the server using the specified number of workers.
 
-    :param app: WSGI callable
-    :param conf: Configuration dictionary
-    """
-    if 'logger' in kwargs:
-        logger = kwargs['logger']
-    else:
-        logger = get_logger(conf, app.log_name)
-    # log uncaught exceptions
-    sys.excepthook = lambda *exc_info: \
-        logger.critical('UNCAUGHT EXCEPTION', exc_info=exc_info)
-    sys.stdout = sys.stderr = LoggerFileObject(logger)
+def get_socket(conf, default_port=8080):
+    """Bind socket to bind ip:port in conf
 
-    try:
-        os.setsid()
-    except OSError:
-        no_cover = True     # pass
+    :param conf: Configuration dict to read settings from
+    :param default_port: port to use if not specified in conf
+
+    :returns : a socket object as returned from socket.listen or
+               ssl.wrap_socket if conf specifies cert_file
+    """
     bind_addr = (conf.get('bind_ip', '0.0.0.0'),
-                 int(conf.get('bind_port', kwargs.get('default_port', 8080))))
+                 int(conf.get('bind_port', default_port)))
+    address_family = [addr[0] for addr in socket.getaddrinfo(bind_addr[0],
+            bind_addr[1], socket.AF_UNSPEC, socket.SOCK_STREAM)
+            if addr[0] in (socket.AF_INET, socket.AF_INET6)][0]
     sock = None
     retry_until = time.time() + 30
     while not sock and time.time() < retry_until:
         try:
-            sock = listen(bind_addr)
+            sock = listen(bind_addr, backlog=int(conf.get('backlog', 4096)),
+                        family=address_family)
             if 'cert_file' in conf:
                 sock = ssl.wrap_socket(sock, certfile=conf['cert_file'],
                     keyfile=conf['key_file'])
@@ -98,11 +91,44 @@ def run_wsgi(app, conf, *args, **kwargs):   # pragma: no cover
     # in my experience, sockets can hang around forever without keepalive
     sock.setsockopt(socket.SOL_SOCKET, socket.SO_KEEPALIVE, 1)
     sock.setsockopt(socket.IPPROTO_TCP, socket.TCP_KEEPIDLE, 600)
-    worker_count = int(conf.get('workers', '1'))
+    return sock
+
+
+# TODO: pull pieces of this out to test
+def run_wsgi(conf_file, app_section, *args, **kwargs):
+    """
+    Loads common settings from conf, then instantiates app and runs
+    the server using the specified number of workers.
+
+    :param conf_file: Path to paste.deploy style configuration file
+    :param app_section: App name from conf file to load config from
+    """
+
+    try:
+        conf = appconfig('config:%s' % conf_file, name=app_section)
+    except Exception, e:
+        print "Error trying to load config %s: %s" % (conf_file, e)
+        return
+    validate_configuration()
+
+    # pre-configure logger
+    log_name = conf.get('log_name', app_section)
+    if 'logger' in kwargs:
+        logger = kwargs.pop('logger')
+    else:
+        logger = get_logger(conf, log_name,
+            log_to_console=kwargs.pop('verbose', False), log_route='wsgi')
+
+    # bind to address and port
+    sock = get_socket(conf, default_port=kwargs.get('default_port', 8080))
+    # remaining tasks should not require elevated privileges
     drop_privileges(conf.get('user', 'swift'))
-    if isinstance(app, type):
-        # Instantiate app if it hasn't been already
-        app = app(conf, *args)
+
+    # finally after binding to ports and privilege drop, run app __init__ code
+    app = loadapp('config:%s' % conf_file, global_conf={'log_name': log_name})
+
+    # redirect errors to logger and close stdio
+    capture_stdio(logger)
 
     def run_server():
         wsgi.HttpProtocol.default_request_version = "HTTP/1.0"
@@ -117,6 +143,7 @@ def run_wsgi(app, conf, *args, **kwargs):   # pragma: no cover
                 raise
         pool.waitall()
 
+    worker_count = int(conf.get('workers', '1'))
     # Useful for profiling [no forks].
     if worker_count == 0:
         run_server()
@@ -146,10 +173,10 @@ def run_wsgi(app, conf, *args, **kwargs):   # pragma: no cover
                 signal.signal(signal.SIGHUP, signal.SIG_DFL)
                 signal.signal(signal.SIGTERM, signal.SIG_DFL)
                 run_server()
-                logger.info('Child %d exiting normally' % os.getpid())
+                logger.notice('Child %d exiting normally' % os.getpid())
                 return
             else:
-                logger.info('Started child %s' % pid)
+                logger.notice('Started child %s' % pid)
                 children.append(pid)
         try:
             pid, status = os.wait()
@@ -159,6 +186,9 @@ def run_wsgi(app, conf, *args, **kwargs):   # pragma: no cover
         except OSError, err:
             if err.errno not in (errno.EINTR, errno.ECHILD):
                 raise
+        except KeyboardInterrupt:
+            logger.notice('User quit')
+            break
     greenio.shutdown_safe(sock)
     sock.close()
-    logger.info('Exited')
+    logger.notice('Exited')
