@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,9 +13,8 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-import os, sys
+import os
 from os.path import isdir, join
-from ConfigParser import ConfigParser
 import random
 import shutil
 import time
@@ -25,22 +24,23 @@ import itertools
 import cPickle as pickle
 
 import eventlet
-from eventlet import GreenPool, tpool, Timeout, sleep
+from eventlet import GreenPool, tpool, Timeout, sleep, hubs
 from eventlet.green import subprocess
 from eventlet.support.greenlets import GreenletExit
 
 from swift.common.ring import Ring
 from swift.common.utils import whataremyips, unlink_older_than, lock_path, \
-        renamer, compute_eta
+        renamer, compute_eta, get_logger
 from swift.common.bufferedhttp import http_connect
+from swift.common.daemon import Daemon
 
+hubs.use_hub('poll')
 
-REPLICAS = 3
-MAX_HANDOFFS = 5
 PICKLE_PROTOCOL = 2
 ONE_WEEK = 604800
 HASH_FILE = 'hashes.pkl'
 
+
 def hash_suffix(path, reclaim_age):
     """
     Performs reclamation and returns an md5 of all (remaining) files.
@@ -61,7 +61,7 @@ def hash_suffix(path, reclaim_age):
         elif files:
             files.sort(reverse=True)
             meta = data = tomb = None
-            for filename in files:
+            for filename in list(files):
                 if not meta and filename.endswith('.meta'):
                     meta = filename
                 if not data and filename.endswith('.data'):
@@ -85,42 +85,14 @@ def hash_suffix(path, reclaim_age):
     return md5.hexdigest()
 
 
-def recalculate_hashes(partition_dir, suffixes, reclaim_age=ONE_WEEK):
-    """
-    Recalculates hashes for the given suffixes in the partition and updates
-    them in the partition's hashes file.
-
-    :param partition_dir: directory of the partition in which to recalculate
-    :param suffixes: list of suffixes to recalculate
-    :param reclaim_age: age in seconds at which tombstones should be removed
-    """
-    def tpool_listdir(partition_dir):
-        return dict(((suff, None) for suff in os.listdir(partition_dir)
-                     if len(suff) == 3 and isdir(join(partition_dir, suff))))
-    hashes_file = join(partition_dir, HASH_FILE)
-    with lock_path(partition_dir):
-        try:
-            with open(hashes_file, 'rb') as fp:
-                hashes = pickle.load(fp)
-        except Exception:
-            hashes = tpool.execute(tpool_listdir, partition_dir)
-        for suffix in suffixes:
-            suffix_dir = join(partition_dir, suffix)
-            if os.path.exists(suffix_dir):
-                hashes[suffix] = hash_suffix(suffix_dir, reclaim_age)
-            elif suffix in hashes:
-                del hashes[suffix]
-        with open(hashes_file + '.tmp', 'wb') as fp:
-            pickle.dump(hashes, fp, PICKLE_PROTOCOL)
-        renamer(hashes_file + '.tmp', hashes_file)
-
-
 def invalidate_hash(suffix_dir):
     """
     Invalidates the hash for a suffix_dir in the partition's hashes file.
 
-    :param suffix_dir: absolute path to suffix dir whose hash needs invalidating
+    :param suffix_dir: absolute path to suffix dir whose hash needs
+                       invalidating
     """
+
     suffix = os.path.basename(suffix_dir)
     partition_dir = os.path.dirname(suffix_dir)
     hashes_file = join(partition_dir, HASH_FILE)
@@ -138,22 +110,21 @@ def invalidate_hash(suffix_dir):
         renamer(hashes_file + '.tmp', hashes_file)
 
 
-def get_hashes(partition_dir, do_listdir=True, reclaim_age=ONE_WEEK):
+def get_hashes(partition_dir, recalculate=[], do_listdir=False,
+               reclaim_age=ONE_WEEK):
     """
     Get a list of hashes for the suffix dir.  do_listdir causes it to mistrust
     the hash cache for suffix existence at the (unexpectedly high) cost of a
     listdir.  reclaim_age is just passed on to hash_suffix.
 
     :param partition_dir: absolute path of partition to get hashes for
+    :param recalculate: list of suffixes which should be recalculated when got
     :param do_listdir: force existence check for all hashes in the partition
     :param reclaim_age: age at which to remove tombstones
 
     :returns: tuple of (number of suffix dirs hashed, dictionary of hashes)
     """
-    def tpool_listdir(hashes, partition_dir):
-        return dict(((suff, hashes.get(suff, None))
-                     for suff in os.listdir(partition_dir)
-                     if len(suff) == 3 and isdir(join(partition_dir, suff))))
+
     hashed = 0
     hashes_file = join(partition_dir, HASH_FILE)
     with lock_path(partition_dir):
@@ -165,8 +136,12 @@ def get_hashes(partition_dir, do_listdir=True, reclaim_age=ONE_WEEK):
         except Exception:
             do_listdir = True
         if do_listdir:
-            hashes = tpool.execute(tpool_listdir, hashes, partition_dir)
+            hashes = dict(((suff, hashes.get(suff, None))
+                       for suff in os.listdir(partition_dir)
+                       if len(suff) == 3 and isdir(join(partition_dir, suff))))
             modified = True
+        for hash_ in recalculate:
+            hashes[hash_] = None
         for suffix, hash_ in hashes.items():
             if not hash_:
                 suffix_dir = join(partition_dir, suffix)
@@ -175,7 +150,7 @@ def get_hashes(partition_dir, do_listdir=True, reclaim_age=ONE_WEEK):
                         hashes[suffix] = hash_suffix(suffix_dir, reclaim_age)
                         hashed += 1
                     except OSError:
-                        logging.exception('Error hashing suffix')
+                        logging.exception(_('Error hashing suffix'))
                         hashes[suffix] = None
                 else:
                     del hashes[suffix]
@@ -188,22 +163,32 @@ def get_hashes(partition_dir, do_listdir=True, reclaim_age=ONE_WEEK):
         return hashed, hashes
 
 
-class ObjectReplicator(object):
+# Hack to work around Eventlet's tpool not catching and reraising Timeouts. We
+# return the Timeout, Timeout if it's raised, the caller looks for it and
+# reraises it if found.
+def tpooled_get_hashes(*args, **kwargs):
+    try:
+        return get_hashes(*args, **kwargs)
+    except Timeout, err:
+        return err, err
+
+
+class ObjectReplicator(Daemon):
     """
     Replicate objects.
 
     Encapsulates most logic and data needed by the object replication process.
-    Each call to .run() performs one replication pass.  It's up to the caller
-    to do this in a loop.
+    Each call to .replicate() performs one replication pass.  It's up to the
+    caller to do this in a loop.
     """
 
-    def __init__(self, conf, logger):
+    def __init__(self, conf):
         """
         :param conf: configuration object obtained from ConfigParser
         :param logger: logging object
         """
         self.conf = conf
-        self.logger = logger
+        self.logger = get_logger(conf, log_route='object-replicator')
         self.devices_dir = conf.get('devices', '/srv/node')
         self.mount_check = conf.get('mount_check', 'true').lower() in \
                               ('true', 't', '1', 'on', 'yes', 'y')
@@ -211,50 +196,59 @@ class ObjectReplicator(object):
                 'vm_test_mode', 'no').lower() in ('yes', 'true', 'on', '1')
         self.swift_dir = conf.get('swift_dir', '/etc/swift')
         self.port = int(conf.get('bind_port', 6000))
-        self.concurrency = int(conf.get('replication_concurrency', 1))
-        self.timeout = conf['timeout']
-        self.stats_interval = int(conf['stats_interval'])
+        self.concurrency = int(conf.get('concurrency', 1))
+        self.stats_interval = int(conf.get('stats_interval', '300'))
         self.object_ring = Ring(join(self.swift_dir, 'object.ring.gz'))
         self.ring_check_interval = int(conf.get('ring_check_interval', 15))
         self.next_check = time.time() + self.ring_check_interval
         self.reclaim_age = int(conf.get('reclaim_age', 86400 * 7))
         self.partition_times = []
+        self.run_pause = int(conf.get('run_pause', 30))
+        self.rsync_timeout = int(conf.get('rsync_timeout', 900))
+        self.rsync_io_timeout = conf.get('rsync_io_timeout', '30')
+        self.http_timeout = int(conf.get('http_timeout', 60))
+        self.lockup_timeout = int(conf.get('lockup_timeout', 1800))
 
     def _rsync(self, args):
         """
         Execute the rsync binary to replicate a partition.
 
-        :returns: a tuple of (rsync exit code, rsync standard output)
+        :returns: return code of rsync process. 0 is successful
         """
         start_time = time.time()
         ret_val = None
         try:
-            with Timeout(120):
-                proc = subprocess.Popen(args, stdout = subprocess.PIPE,
-                    stderr = subprocess.STDOUT)
+            with Timeout(self.rsync_timeout):
+                proc = subprocess.Popen(args, stdout=subprocess.PIPE,
+                    stderr=subprocess.STDOUT)
                 results = proc.stdout.read()
                 ret_val = proc.wait()
-        finally:
-            if ret_val is None:
-                proc.kill()
+        except Timeout:
+            self.logger.error(_("Killing long-running rsync: %s"), str(args))
+            proc.kill()
+            return 1  # failure response code
         total_time = time.time() - start_time
-        if results:
-            for result in results.split('\n'):
-                if result == '':
-                    continue
-                if result.startswith('cd+'):
-                    continue
+        for result in results.split('\n'):
+            if result == '':
+                continue
+            if result.startswith('cd+'):
+                continue
+            if not ret_val:
                 self.logger.info(result)
+            else:
+                self.logger.error(result)
+        if ret_val:
+            self.logger.error(_('Bad rsync return code: %(args)s -> %(ret)d'),
+                    {'args': str(args), 'ret': ret_val})
+        elif results:
             self.logger.info(
-                "Sync of %s at %s complete (%.03f) [%d]" % (
-                args[-2], args[-1], total_time, ret_val))
+                _("Successful rsync of %(src)s at %(dst)s (%(time).03f)"),
+                {'src': args[-2], 'dst': args[-1], 'time': total_time})
         else:
             self.logger.debug(
-                "Sync of %s at %s complete (%.03f) [%d]" % (
-                args[-2], args[-1], total_time, ret_val))
-        if ret_val:
-            self.logger.error('Bad rsync return code: %d' % ret_val)
-        return ret_val, results
+                _("Successful rsync of %(src)s at %(dst)s (%(time).03f)"),
+                {'src': args[-2], 'dst': args[-1], 'time': total_time})
+        return ret_val
 
     def rsync(self, node, job, suffixes):
         """
@@ -277,8 +271,8 @@ class ObjectReplicator(object):
             '--xattrs',
             '--itemize-changes',
             '--ignore-existing',
-            '--timeout=%s' % self.timeout,
-            '--contimeout=%s' % self.timeout,
+            '--timeout=%s' % self.rsync_io_timeout,
+            '--contimeout=%s' % self.rsync_io_timeout,
         ]
         if self.vm_test_mode:
             rsync_module = '%s::object%s' % (node['ip'], node['port'])
@@ -294,8 +288,7 @@ class ObjectReplicator(object):
             return False
         args.append(join(rsync_module, node['device'],
                     'objects', job['partition']))
-        ret_val, results = self._rsync(args)
-        return ret_val == 0
+        return self._rsync(args) == 0
 
     def check_ring(self):
         """
@@ -311,11 +304,12 @@ class ObjectReplicator(object):
 
     def update_deleted(self, job):
         """
-        High-level method that replicates a single partition that doesn't belong
-        on this node.
+        High-level method that replicates a single partition that doesn't
+        belong on this node.
 
         :param job: a dict containing info about the partition to be replicated
         """
+
         def tpool_get_suffixes(path):
             return [suff for suff in os.listdir(path)
                     if len(suff) == 3 and isdir(join(path, suff))]
@@ -328,18 +322,18 @@ class ObjectReplicator(object):
                 for node in job['nodes']:
                     success = self.rsync(node, job, suffixes)
                     if success:
-                        with Timeout(60):
+                        with Timeout(self.http_timeout):
                             http_connect(node['ip'], node['port'],
                                 node['device'], job['partition'], 'REPLICATE',
                                 '/' + '-'.join(suffixes),
-                                headers={'Content-Length': '0'}
-                            ).getresponse().read()
+                          headers={'Content-Length': '0'}).getresponse().read()
                     responses.append(success)
-            if not suffixes or (len(responses) == REPLICAS and all(responses)):
-                self.logger.info("Removing partition: %s" % job['path'])
+            if not suffixes or (len(responses) == \
+                        self.object_ring.replica_count and all(responses)):
+                self.logger.info(_("Removing partition: %s"), job['path'])
                 tpool.execute(shutil.rmtree, job['path'], ignore_errors=True)
         except (Exception, Timeout):
-            self.logger.exception("Error syncing handoff partition")
+            self.logger.exception(_("Error syncing handoff partition"))
         finally:
             self.partition_times.append(time.time() - begin)
 
@@ -352,47 +346,63 @@ class ObjectReplicator(object):
         self.replication_count += 1
         begin = time.time()
         try:
-            hashed, local_hash = get_hashes(job['path'],
+            hashed, local_hash = tpool.execute(tpooled_get_hashes, job['path'],
                     do_listdir=(self.replication_count % 10) == 0,
                     reclaim_age=self.reclaim_age)
+            # See tpooled_get_hashes "Hack".
+            if isinstance(hashed, BaseException):
+                raise hashed
             self.suffix_hash += hashed
-            successes = 0
+            attempts_left = self.object_ring.replica_count - 1
             nodes = itertools.chain(job['nodes'],
                         self.object_ring.get_more_nodes(int(job['partition'])))
-            while successes < (REPLICAS - 1):
+            while attempts_left > 0:
+                # If this throws StopIterator it will be caught way below
                 node = next(nodes)
+                attempts_left -= 1
                 try:
-                    with Timeout(60):
+                    with Timeout(self.http_timeout):
                         resp = http_connect(node['ip'], node['port'],
                                 node['device'], job['partition'], 'REPLICATE',
-                                '', headers={'Content-Length': '0'}
-                        ).getresponse()
+                            '', headers={'Content-Length': '0'}).getresponse()
+                        if resp.status == 507:
+                            self.logger.error(_('%(ip)s/%(device)s responded'
+                                    ' as unmounted'), node)
+                            attempts_left += 1
+                            continue
                         if resp.status != 200:
-                            self.logger.error("Invalid response %s from %s" %
-                                    (resp.status, node['ip']))
+                            self.logger.error(_("Invalid response %(resp)s "
+                                "from %(ip)s"),
+                                {'resp': resp.status, 'ip': node['ip']})
                             continue
                         remote_hash = pickle.loads(resp.read())
                         del resp
-                    successes += 1
-                    suffixes = [suffix for suffix in local_hash
-                           if local_hash[suffix] != remote_hash.get(suffix, -1)]
+                    suffixes = [suffix for suffix in local_hash if
+                            local_hash[suffix] != remote_hash.get(suffix, -1)]
                     if not suffixes:
                         continue
-                    success = self.rsync(node, job, suffixes)
-                    recalculate_hashes(job['path'], suffixes,
-                                       reclaim_age=self.reclaim_age)
-                    with Timeout(60):
-                        http_connect(node['ip'], node['port'],
+                    hashed, local_hash = tpool.execute(tpooled_get_hashes,
+                        job['path'], recalculate=suffixes,
+                        reclaim_age=self.reclaim_age)
+                    # See tpooled_get_hashes "Hack".
+                    if isinstance(hashed, BaseException):
+                        raise hashed
+                    suffixes = [suffix for suffix in local_hash if
+                            local_hash[suffix] != remote_hash.get(suffix, -1)]
+                    self.rsync(node, job, suffixes)
+                    with Timeout(self.http_timeout):
+                        conn = http_connect(node['ip'], node['port'],
                             node['device'], job['partition'], 'REPLICATE',
                             '/' + '-'.join(suffixes),
-                            headers={'Content-Length': '0'}
-                        ).getresponse().read()
+                            headers={'Content-Length': '0'})
+                        conn.getresponse().read()
                     self.suffix_sync += len(suffixes)
                 except (Exception, Timeout):
-                    logging.exception("Error syncing with node: %s" % node)
+                    self.logger.exception(_("Error syncing with node: %s") %
+                                            node)
             self.suffix_count += len(local_hash)
         except (Exception, Timeout):
-            self.logger.exception("Error syncing partition")
+            self.logger.exception(_("Error syncing partition"))
         finally:
             self.partition_times.append(time.time() - begin)
 
@@ -402,23 +412,30 @@ class ObjectReplicator(object):
         """
         if self.replication_count:
             rate = self.replication_count / (time.time() - self.start)
-            left = int((self.job_count - self.replication_count) / rate)
-            self.logger.info("%d/%d (%.2f%%) partitions replicated in %.2f seconds (%.2f/sec, %s remaining)"
-                    % (self.replication_count, self.job_count,
-                       self.replication_count * 100.0 / self.job_count,
-                       time.time() - self.start, rate,
-                       '%d%s' % compute_eta(self.start, self.replication_count, self.job_count)))
+            self.logger.info(_("%(replicated)d/%(total)d (%(percentage).2f%%)"
+                " partitions replicated in %(time).2fs (%(rate).2f/sec, "
+                "%(remaining)s remaining)"),
+                {'replicated': self.replication_count, 'total': self.job_count,
+                 'percentage': self.replication_count * 100.0 / self.job_count,
+                 'time': time.time() - self.start, 'rate': rate,
+                 'remaining': '%d%s' % compute_eta(self.start,
+                           self.replication_count, self.job_count)})
             if self.suffix_count:
-                self.logger.info("%d suffixes checked - %.2f%% hashed, %.2f%% synced" %
-                    (self.suffix_count,
-                     (self.suffix_hash * 100.0) / self.suffix_count,
-                     (self.suffix_sync * 100.0) / self.suffix_count))
+                self.logger.info(_("%(checked)d suffixes checked - "
+                    "%(hashed).2f%% hashed, %(synced).2f%% synced"),
+                    {'checked': self.suffix_count,
+                     'hashed': (self.suffix_hash * 100.0) / self.suffix_count,
+                     'synced': (self.suffix_sync * 100.0) / self.suffix_count})
                 self.partition_times.sort()
-                self.logger.info("Partition times: max %.4fs, min %.4fs, med %.4fs"
-                    % (self.partition_times[-1], self.partition_times[0],
-                       self.partition_times[len(self.partition_times) // 2]))
+                self.logger.info(_("Partition times: max %(max).4fs, "
+                    "min %(min).4fs, med %(med).4fs"),
+                    {'max': self.partition_times[-1],
+                     'min': self.partition_times[0],
+                     'med': self.partition_times[
+                                len(self.partition_times) // 2]})
         else:
-            self.logger.info("Nothing replicated for %s seconds." % (time.time() - self.start))
+            self.logger.info(_("Nothing replicated for %s seconds."),
+                (time.time() - self.start))
 
     def kill_coros(self):
         """Utility function that kills all coroutines currently running."""
@@ -431,18 +448,61 @@ class ObjectReplicator(object):
     def heartbeat(self):
         """
         Loop that runs in the background during replication.  It periodically
-        logs progress and attempts to detect lockups, killing any running
-        coroutines if the replicator hasn't made progress since last hearbeat.
+        logs progress.
         """
         while True:
-            if self.replication_count == self.last_replication_count:
-                self.logger.error("Lockup detected.. killing live coros.")
-                self.kill_coros()
-            self.last_replication_count = self.replication_count
-            eventlet.sleep(300)
+            eventlet.sleep(self.stats_interval)
             self.stats_line()
 
-    def run(self):
+    def detect_lockups(self):
+        """
+        In testing, the pool.waitall() call very occasionally failed to return.
+        This is an attempt to make sure the replicator finishes its replication
+        pass in some eventuality.
+        """
+        while True:
+            eventlet.sleep(self.lockup_timeout)
+            if self.replication_count == self.last_replication_count:
+                self.logger.error(_("Lockup detected.. killing live coros."))
+                self.kill_coros()
+            self.last_replication_count = self.replication_count
+
+    def collect_jobs(self):
+        """
+        Returns a sorted list of jobs (dictionaries) that specify the
+        partitions, nodes, etc to be rsynced.
+        """
+        jobs = []
+        ips = whataremyips()
+        for local_dev in [dev for dev in self.object_ring.devs
+                if dev and dev['ip'] in ips and dev['port'] == self.port]:
+            dev_path = join(self.devices_dir, local_dev['device'])
+            obj_path = join(dev_path, 'objects')
+            tmp_path = join(dev_path, 'tmp')
+            if self.mount_check and not os.path.ismount(dev_path):
+                self.logger.warn(_('%s is not mounted'), local_dev['device'])
+                continue
+            unlink_older_than(tmp_path, time.time() - self.reclaim_age)
+            if not os.path.exists(obj_path):
+                continue
+            for partition in os.listdir(obj_path):
+                try:
+                    nodes = [node for node in
+                        self.object_ring.get_part_nodes(int(partition))
+                             if node['id'] != local_dev['id']]
+                    jobs.append(dict(path=join(obj_path, partition),
+                        nodes=nodes,
+                        delete=len(nodes) > self.object_ring.replica_count - 1,
+                        partition=partition))
+                except ValueError:
+                    continue
+        random.shuffle(jobs)
+        # Partititons that need to be deleted take priority
+        jobs.sort(key=lambda job: not job['delete'])
+        self.job_count = len(jobs)
+        return jobs
+
+    def replicate(self):
         """Run a replication pass"""
         self.start = time.time()
         self.suffix_count = 0
@@ -451,51 +511,49 @@ class ObjectReplicator(object):
         self.replication_count = 0
         self.last_replication_count = -1
         self.partition_times = []
-        jobs = []
         stats = eventlet.spawn(self.heartbeat)
+        lockup_detector = eventlet.spawn(self.detect_lockups)
         try:
-            ips = whataremyips()
             self.run_pool = GreenPool(size=self.concurrency)
-            for local_dev in [
-                dev for dev in self.object_ring.devs
-                if dev and dev['ip'] in ips and dev['port'] == self.port
-            ]:
-                dev_path = join(self.devices_dir, local_dev['device'])
-                obj_path = join(dev_path, 'objects')
-                tmp_path = join(dev_path, 'tmp')
-                if self.mount_check and not os.path.ismount(dev_path):
-                    self.logger.warn('%s is not mounted' % local_dev['device'])
-                    continue
-                unlink_older_than(tmp_path, time.time() - self.reclaim_age)
-                if not os.path.exists(obj_path):
-                    continue
-                for partition in os.listdir(obj_path):
-                    try:
-                        nodes = [node for node in
-                                 self.object_ring.get_part_nodes(int(partition))
-                                 if node['id'] != local_dev['id']]
-                        jobs.append(dict(path=join(obj_path, partition),
-                            nodes=nodes, delete=len(nodes) > 2,
-                            partition=partition))
-                    except ValueError:
-                        continue
-            random.shuffle(jobs)
-            # Partititons that need to be deleted take priority
-            jobs.sort(key=lambda job: not job['delete'])
-            self.job_count = len(jobs)
+            jobs = self.collect_jobs()
             for job in jobs:
                 if not self.check_ring():
-                    self.logger.info(
-                    "Ring change detected. Aborting current replication pass.")
+                    self.logger.info(_("Ring change detected. Aborting "
+                            "current replication pass."))
                     return
                 if job['delete']:
                     self.run_pool.spawn(self.update_deleted, job)
                 else:
                     self.run_pool.spawn(self.update, job)
-            with Timeout(120):
+            with Timeout(self.lockup_timeout):
                 self.run_pool.waitall()
         except (Exception, Timeout):
-            self.logger.exception("Exception while replicating")
+            self.logger.exception(_("Exception in top-level replication loop"))
             self.kill_coros()
-        self.stats_line()
-        stats.kill()
+        finally:
+            stats.kill()
+            lockup_detector.kill()
+            self.stats_line()
+
+    def run_once(self, *args, **kwargs):
+        start = time.time()
+        self.logger.info(_("Running object replicator in script mode."))
+        self.replicate()
+        total = (time.time() - start) / 60
+        self.logger.info(
+            _("Object replication complete. (%.02f minutes)"), total)
+
+    def run_forever(self, *args, **kwargs):
+        self.logger.info(_("Starting object replicator in daemon mode."))
+        # Run the replicator continually
+        while True:
+            start = time.time()
+            self.logger.info(_("Starting object replication pass."))
+            # Run the replicator
+            self.replicate()
+            total = (time.time() - start) / 60
+            self.logger.info(
+                _("Object replication complete. (%.02f minutes)"), total)
+            self.logger.debug(_('Replication sleeping for %s seconds.'),
+                self.run_pause)
+            sleep(self.run_pause)
