@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,9 +19,9 @@ from __future__ import with_statement
 import cPickle as pickle
 import errno
 import os
-import socket
 import time
 import traceback
+import uuid
 from datetime import datetime
 from hashlib import md5
 from tempfile import mkstemp
@@ -30,23 +30,21 @@ from contextlib import contextmanager
 
 from webob import Request, Response, UTC
 from webob.exc import HTTPAccepted, HTTPBadRequest, HTTPCreated, \
-    HTTPInternalServerError, HTTPLengthRequired, HTTPNoContent, HTTPNotFound, \
-    HTTPNotImplemented, HTTPNotModified, HTTPPreconditionFailed, \
+    HTTPInternalServerError, HTTPNoContent, HTTPNotFound, \
+    HTTPNotModified, HTTPPreconditionFailed, \
     HTTPRequestTimeout, HTTPUnprocessableEntity, HTTPMethodNotAllowed
 from xattr import getxattr, setxattr
-from eventlet import sleep, Timeout
+from eventlet import sleep, Timeout, TimeoutError, tpool
 
-from swift.common.exceptions import MessageTimeout
 from swift.common.utils import mkdirs, normalize_timestamp, \
-    storage_directory, hash_path, get_logger, renamer, fallocate, \
-    split_path, drop_buffer_cache
-from swift.common.healthcheck import healthcheck
+    storage_directory, hash_path, renamer, fallocate, \
+    split_path, drop_buffer_cache, get_logger, write_pickle
 from swift.common.bufferedhttp import http_connect
 from swift.common.constraints import check_object_creation, check_mount, \
-    check_float, check_xml_encodable
-from swift.common.exceptions import ConnectionTimeout
-from swift.obj.replicator import get_hashes, invalidate_hash, \
-    recalculate_hashes
+    check_float, check_utf8
+from swift.common.exceptions import ConnectionTimeout, DiskFileError, \
+    DiskFileNotExist
+from swift.obj.replicator import tpooled_get_hashes, invalidate_hash
 
 
 DATADIR = 'objects'
@@ -54,6 +52,9 @@ ASYNCDIR = 'async_pending'
 PICKLE_PROTOCOL = 2
 METADATA_KEY = 'user.swift.metadata'
 MAX_OBJECT_NAME_LENGTH = 1024
+KEEP_CACHE_SIZE = (5 * 1024 * 1024)
+# keep these lower-case
+DISALLOWED_HEADERS = set('content-length content-type deleted etag'.split())
 
 
 def read_metadata(fd):
@@ -75,6 +76,47 @@ def read_metadata(fd):
     return pickle.loads(metadata)
 
 
+def write_metadata(fd, metadata):
+    """
+    Helper function to write pickled metadata for an object file.
+
+    :param fd: file descriptor to write the metadata
+    :param metadata: metadata to write
+    """
+    metastr = pickle.dumps(metadata, PICKLE_PROTOCOL)
+    key = 0
+    while metastr:
+        setxattr(fd, '%s%s' % (METADATA_KEY, key or ''), metastr[:254])
+        metastr = metastr[254:]
+        key += 1
+
+
+def quarantine_renamer(device_path, corrupted_file_path):
+    """
+    In the case that a file is corrupted, move it to a quarantined
+    area to allow replication to fix it.
+
+    :params device_path: The path to the device the corrupted file is on.
+    :params corrupted_file_path: The path to the file you want quarantined.
+
+    :returns: path (str) of directory the file was moved to
+    :raises OSError: re-raises non errno.EEXIST / errno.ENOTEMPTY
+                     exceptions from rename
+    """
+    from_dir = os.path.dirname(corrupted_file_path)
+    to_dir = os.path.join(device_path, 'quarantined',
+                          'objects', os.path.basename(from_dir))
+    invalidate_hash(os.path.dirname(from_dir))
+    try:
+        renamer(from_dir, to_dir)
+    except OSError, e:
+        if e.errno not in (errno.EEXIST, errno.ENOTEMPTY):
+            raise
+        to_dir = "%s-%s" % (to_dir, uuid.uuid4().hex)
+        renamer(from_dir, to_dir)
+    return to_dir
+
+
 class DiskFile(object):
     """
     Manage object files on disk.
@@ -88,17 +130,26 @@ class DiskFile(object):
     :param keep_data_fp: if True, don't close the fp, otherwise close it
     :param disk_chunk_Size: size of chunks on file reads
     """
+
     def __init__(self, path, device, partition, account, container, obj,
-                    keep_data_fp=False, disk_chunk_size=65536):
+                 logger, keep_data_fp=False, disk_chunk_size=65536):
         self.disk_chunk_size = disk_chunk_size
         self.name = '/' + '/'.join((account, container, obj))
         name_hash = hash_path(account, container, obj)
         self.datadir = os.path.join(path, device,
                     storage_directory(DATADIR, partition, name_hash))
+        self.device_path = os.path.join(path, device)
         self.tmpdir = os.path.join(path, device, 'tmp')
+        self.logger = logger
         self.metadata = {}
         self.meta_file = None
         self.data_file = None
+        self.fp = None
+        self.iter_etag = None
+        self.started_at_0 = False
+        self.read_to_eof = False
+        self.quarantined_dir = None
+        self.keep_cache = False
         if not os.path.exists(self.datadir):
             return
         files = sorted(os.listdir(self.datadir), reverse=True)
@@ -117,12 +168,11 @@ class DiskFile(object):
         self.fp = open(self.data_file, 'rb')
         self.metadata = read_metadata(self.fp)
         if not keep_data_fp:
-            self.close()
+            self.close(verify_file=False)
         if self.meta_file:
             with open(self.meta_file) as mfp:
                 for key in self.metadata.keys():
-                    if key.lower() not in ('content-type', 'content-encoding',
-                                'deleted', 'content-length', 'etag'):
+                    if key.lower() not in DISALLOWED_HEADERS:
                         del self.metadata[key]
                 self.metadata.update(read_metadata(mfp))
 
@@ -131,17 +181,25 @@ class DiskFile(object):
         try:
             dropped_cache = 0
             read = 0
+            self.started_at_0 = False
+            self.read_to_eof = False
+            if self.fp.tell() == 0:
+                self.started_at_0 = True
+                self.iter_etag = md5()
             while True:
                 chunk = self.fp.read(self.disk_chunk_size)
                 if chunk:
+                    if self.iter_etag:
+                        self.iter_etag.update(chunk)
                     read += len(chunk)
                     if read - dropped_cache > (1024 * 1024):
-                        drop_buffer_cache(self.fp.fileno(), dropped_cache,
+                        self.drop_cache(self.fp.fileno(), dropped_cache,
                             read - dropped_cache)
                         dropped_cache = read
                     yield chunk
                 else:
-                    drop_buffer_cache(self.fp.fileno(), dropped_cache,
+                    self.read_to_eof = True
+                    self.drop_cache(self.fp.fileno(), dropped_cache,
                         read - dropped_cache)
                     break
         finally:
@@ -164,11 +222,41 @@ class DiskFile(object):
                     break
             yield chunk
 
-    def close(self):
-        """Close the file."""
+    def _handle_close_quarantine(self):
+        """Check if file needs to be quarantined"""
+        try:
+            obj_size = self.get_data_file_size()
+        except DiskFileError, e:
+            self.quarantine()
+            return
+        except DiskFileNotExist:
+            return
+
+        if (self.iter_etag and self.started_at_0 and self.read_to_eof and
+            'ETag' in self.metadata and
+            self.iter_etag.hexdigest() != self.metadata.get('ETag')):
+                self.quarantine()
+
+    def close(self, verify_file=True):
+        """
+        Close the file. Will handle quarantining file if necessary.
+
+        :param verify_file: Defaults to True. If false, will not check
+                            file to see if it needs quarantining.
+        """
         if self.fp:
-            self.fp.close()
-            self.fp = None
+            try:
+                if verify_file:
+                    self._handle_close_quarantine()
+            except Exception, e:
+                import traceback
+                self.logger.error(_('ERROR DiskFile %(data_file)s in '
+                     '%(data_dir)s close failure: %(exc)s : %(stack)'),
+                     {'exc': e, 'stack': ''.join(traceback.format_stack()),
+                      'data_file': self.data_file, 'data_dir': self.datadir})
+            finally:
+                self.fp.close()
+                self.fp = None
 
     def is_deleted(self):
         """
@@ -205,20 +293,15 @@ class DiskFile(object):
 
         :params fd: file descriptor of the temp file
         :param tmppath: path to the temporary file being used
-        :param metadata: dictionary of metada to be written
+        :param metadata: dictionary of metadata to be written
         :param extention: extension to be used when making the file
         """
         metadata['name'] = self.name
         timestamp = normalize_timestamp(metadata['X-Timestamp'])
-        metastr = pickle.dumps(metadata, PICKLE_PROTOCOL)
-        key = 0
-        while metastr:
-            setxattr(fd, '%s%s' % (METADATA_KEY, key or ''), metastr[:254])
-            metastr = metastr[254:]
-            key += 1
+        write_metadata(fd, metadata)
         if 'Content-Length' in metadata:
-            drop_buffer_cache(fd, 0, int(metadata['Content-Length']))
-        os.fsync(fd)
+            self.drop_cache(fd, 0, int(metadata['Content-Length']))
+        tpool.execute(os.fsync, fd)
         invalidate_hash(os.path.dirname(self.datadir))
         renamer(tmppath, os.path.join(self.datadir, timestamp + extension))
         self.metadata = metadata
@@ -239,12 +322,53 @@ class DiskFile(object):
                     if err.errno != errno.ENOENT:
                         raise
 
+    def drop_cache(self, fd, offset, length):
+        """Method for no-oping buffer cache drop method."""
+        if not self.keep_cache:
+            drop_buffer_cache(fd, offset, length)
+
+    def quarantine(self):
+        """
+        In the case that a file is corrupted, move it to a quarantined
+        area to allow replication to fix it.
+
+        :returns: if quarantine is successful, path to quarantined
+                  directory otherwise None
+        """
+        if not (self.is_deleted() or self.quarantined_dir):
+            self.quarantined_dir = quarantine_renamer(self.device_path,
+                                                      self.data_file)
+            return self.quarantined_dir
+
+    def get_data_file_size(self):
+        """
+        Returns the os.path.getsize for the file.  Raises an exception if this
+        file does not match the Content-Length stored in the metadata. Or if
+        self.data_file does not exist.
+
+        :returns: file size as an int
+        :raises DiskFileError: on file size mismatch.
+        :raises DiskFileNotExist: on file not existing (including deleted)
+        """
+        try:
+            file_size = 0
+            if self.data_file:
+                file_size = os.path.getsize(self.data_file)
+                if 'Content-Length' in self.metadata:
+                    metadata_size = int(self.metadata['Content-Length'])
+                    if file_size != metadata_size:
+                        raise DiskFileError('Content-Length of %s does not '
+                          'match file size of %s' % (metadata_size, file_size))
+                return file_size
+        except OSError, err:
+            if err.errno != errno.ENOENT:
+                raise
+        raise DiskFileNotExist('Data File does not exist.')
+
 
 class ObjectController(object):
     """Implements the WSGI application for the Swift Object Server."""
 
-    log_name = 'object'
-
     def __init__(self, conf):
         """
         Creates a new WSGI application for the Swift Object Server. An
@@ -252,7 +376,7 @@ class ObjectController(object):
         <source-dir>/etc/object-server.conf-sample or
         /etc/swift/object-server.conf-sample.
         """
-        self.logger = get_logger(conf, self.log_name)
+        self.logger = get_logger(conf, log_route='object-server')
         self.devices = conf.get('devices', '/srv/node/')
         self.mount_check = conf.get('mount_check', 'true').lower() in \
                               ('true', 't', '1', 'on', 'yes', 'y')
@@ -263,7 +387,13 @@ class ObjectController(object):
         self.log_requests = conf.get('log_requests', 't')[:1].lower() == 't'
         self.max_upload_time = int(conf.get('max_upload_time', 86400))
         self.slow = int(conf.get('slow', 0))
-        self.chunks_per_sync = int(conf.get('chunks_per_sync', 8000))
+        self.bytes_per_sync = int(conf.get('mb_per_sync', 512)) * 1024 * 1024
+        default_allowed_headers = 'content-encoding, x-object-manifest, ' \
+                                  'content-disposition'
+        self.allowed_headers = set(i.strip().lower() for i in \
+                conf.get('allowed_headers', \
+                default_allowed_headers).split(',') if i.strip() and \
+                i.strip().lower() not in DISALLOWED_HEADERS)
 
     def container_update(self, op, account, container, obj, headers_in,
                          headers_out, objdevice):
@@ -287,7 +417,7 @@ class ObjectController(object):
         full_path = '/%s/%s/%s' % (account, container, obj)
         try:
             with ConnectionTimeout(self.conn_timeout):
-                ip, port = host.split(':')
+                ip, port = host.rsplit(':', 1)
                 conn = http_connect(ip, port, contdevice, partition, op,
                         full_path, headers_out)
             with Timeout(self.node_timeout):
@@ -296,23 +426,23 @@ class ObjectController(object):
                 if 200 <= response.status < 300:
                     return
                 else:
-                    self.logger.error('ERROR Container update failed (saving '
-                        'for async update later): %d response from %s:%s/%s' %
-                        (response.status, ip, port, contdevice))
-        except:
-            self.logger.exception('ERROR container update failed with '
-                '%s:%s/%s transaction %s (saving for async update later)' %
-                (ip, port, contdevice, headers_in.get('x-cf-trans-id', '-')))
+                    self.logger.error(_('ERROR Container update failed '
+                        '(saving for async update later): %(status)d '
+                        'response from %(ip)s:%(port)s/%(dev)s'),
+                        {'status': response.status, 'ip': ip, 'port': port,
+                         'dev': contdevice})
+        except (Exception, TimeoutError):
+            self.logger.exception(_('ERROR container update failed with '
+                '%(ip)s:%(port)s/%(dev)s (saving for async update later)'),
+                {'ip': ip, 'port': port, 'dev': contdevice})
         async_dir = os.path.join(self.devices, objdevice, ASYNCDIR)
-        fd, tmppath = mkstemp(dir=os.path.join(self.devices, objdevice, 'tmp'))
-        with os.fdopen(fd, 'wb') as fo:
-            pickle.dump({'op': op, 'account': account, 'container': container,
-                         'obj': obj, 'headers': headers_out}, fo)
-            fo.flush()
-            os.fsync(fd)
-            ohash = hash_path(account, container, obj)
-            renamer(tmppath, os.path.join(async_dir, ohash[-3:], ohash + '-' +
-                normalize_timestamp(headers_out['x-timestamp'])))
+        ohash = hash_path(account, container, obj)
+        write_pickle(
+            {'op': op, 'account': account, 'container': container,
+                'obj': obj, 'headers': headers_out},
+            os.path.join(async_dir, ohash[-3:], ohash + '-' +
+                normalize_timestamp(headers_out['x-timestamp'])),
+            os.path.join(self.devices, objdevice, 'tmp'))
 
     def POST(self, request):
         """Handle HTTP POST requests for the Swift Object Server."""
@@ -329,16 +459,24 @@ class ObjectController(object):
         if self.mount_check and not check_mount(self.devices, device):
             return Response(status='507 %s is not mounted' % device)
         file = DiskFile(self.devices, device, partition, account, container,
-                        obj, disk_chunk_size=self.disk_chunk_size)
-        deleted = file.is_deleted()
+                        obj, self.logger, disk_chunk_size=self.disk_chunk_size)
+
         if file.is_deleted():
             response_class = HTTPNotFound
         else:
             response_class = HTTPAccepted
-        old_metadata = file.metadata
+        try:
+            file_size = file.get_data_file_size()
+        except (DiskFileError, DiskFileNotExist):
+            file.quarantine()
+            return HTTPNotFound(request=request)
         metadata = {'X-Timestamp': request.headers['x-timestamp']}
         metadata.update(val for val in request.headers.iteritems()
                 if val[0].lower().startswith('x-object-meta-'))
+        for header_key in self.allowed_headers:
+            if header_key in request.headers:
+                header_caps = header_key.title()
+                metadata[header_caps] = request.headers[header_key]
         with file.mkstemp() as (fd, tmppath):
             file.put(fd, tmppath, metadata, extension='.meta')
         return response_class(request=request)
@@ -361,17 +499,16 @@ class ObjectController(object):
         if error_response:
             return error_response
         file = DiskFile(self.devices, device, partition, account, container,
-                        obj, disk_chunk_size=self.disk_chunk_size)
+                        obj, self.logger, disk_chunk_size=self.disk_chunk_size)
         upload_expiration = time.time() + self.max_upload_time
         etag = md5()
         upload_size = 0
+        last_sync = 0
         with file.mkstemp() as (fd, tmppath):
             if 'content-length' in request.headers:
                 fallocate(fd, int(request.headers['content-length']))
-            chunk_count = 0
-            dropped_cache = 0
-            for chunk in iter(lambda: request.body_file.read(
-                    self.network_chunk_size), ''):
+            reader = request.environ['wsgi.input'].read
+            for chunk in iter(lambda: reader(self.network_chunk_size), ''):
                 upload_size += len(chunk)
                 if time.time() > upload_expiration:
                     return HTTPRequestTimeout(request=request)
@@ -379,13 +516,11 @@ class ObjectController(object):
                 while chunk:
                     written = os.write(fd, chunk)
                     chunk = chunk[written:]
-                chunk_count += 1
                 # For large files sync every 512MB (by default) written
-                if chunk_count % self.chunks_per_sync == 0:
-                    os.fdatasync(fd)
-                    drop_buffer_cache(fd, dropped_cache,
-                        upload_size - dropped_cache)
-                    dropped_cache = upload_size
+                if upload_size - last_sync >= self.bytes_per_sync:
+                    tpool.execute(os.fdatasync, fd)
+                    drop_buffer_cache(fd, last_sync, upload_size - last_sync)
+                    last_sync = upload_size
 
             if 'content-length' in request.headers and \
                     int(request.headers['content-length']) != upload_size:
@@ -403,9 +538,10 @@ class ObjectController(object):
             metadata.update(val for val in request.headers.iteritems()
                     if val[0].lower().startswith('x-object-meta-') and
                     len(val[0]) > 14)
-            if 'content-encoding' in request.headers:
-                metadata['Content-Encoding'] = \
-                    request.headers['Content-Encoding']
+            for header_key in self.allowed_headers:
+                if header_key in request.headers:
+                    header_caps = header_key.title()
+                    metadata[header_caps] = request.headers[header_key]
             file.put(fd, tmppath, metadata)
         file.unlinkold(metadata['X-Timestamp'])
         self.container_update('PUT', account, container, obj, request.headers,
@@ -429,12 +565,18 @@ class ObjectController(object):
         if self.mount_check and not check_mount(self.devices, device):
             return Response(status='507 %s is not mounted' % device)
         file = DiskFile(self.devices, device, partition, account, container,
-                obj, keep_data_fp=True, disk_chunk_size=self.disk_chunk_size)
+                        obj, self.logger, keep_data_fp=True,
+                        disk_chunk_size=self.disk_chunk_size)
         if file.is_deleted():
             if request.headers.get('if-match') == '*':
                 return HTTPPreconditionFailed(request=request)
             else:
                 return HTTPNotFound(request=request)
+        try:
+            file_size = file.get_data_file_size()
+        except (DiskFileError, DiskFileNotExist):
+            file.quarantine()
+            return HTTPNotFound(request=request)
         if request.headers.get('if-match') not in (None, '*') and \
                 file.metadata['ETag'] not in request.if_match:
             file.close()
@@ -469,11 +611,16 @@ class ObjectController(object):
                         'application/octet-stream'), app_iter=file,
                         request=request, conditional_response=True)
         for key, value in file.metadata.iteritems():
-            if key.lower().startswith('x-object-meta-'):
+            if key.lower().startswith('x-object-meta-') or \
+                    key.lower() in self.allowed_headers:
                 response.headers[key] = value
         response.etag = file.metadata['ETag']
         response.last_modified = float(file.metadata['X-Timestamp'])
-        response.content_length = int(file.metadata['Content-Length'])
+        response.content_length = file_size
+        if response.content_length < KEEP_CACHE_SIZE and \
+                'X-Auth-Token' not in request.headers and \
+                'X-Storage-Token' not in request.headers:
+            file.keep_cache = True
         if 'Content-Encoding' in file.metadata:
             response.content_encoding = file.metadata['Content-Encoding']
         return request.get_response(response)
@@ -491,17 +638,23 @@ class ObjectController(object):
         if self.mount_check and not check_mount(self.devices, device):
             return Response(status='507 %s is not mounted' % device)
         file = DiskFile(self.devices, device, partition, account, container,
-                        obj, disk_chunk_size=self.disk_chunk_size)
+                        obj, self.logger, disk_chunk_size=self.disk_chunk_size)
         if file.is_deleted():
             return HTTPNotFound(request=request)
+        try:
+            file_size = file.get_data_file_size()
+        except (DiskFileError, DiskFileNotExist):
+            file.quarantine()
+            return HTTPNotFound(request=request)
         response = Response(content_type=file.metadata['Content-Type'],
                             request=request, conditional_response=True)
         for key, value in file.metadata.iteritems():
-            if key.lower().startswith('x-object-meta-'):
+            if key == 'X-Object-Manifest' or \
+                    key.lower().startswith('x-object-meta-'):
                 response.headers[key] = value
         response.etag = file.metadata['ETag']
         response.last_modified = float(file.metadata['X-Timestamp'])
-        response.content_length = int(file.metadata['Content-Length'])
+        response.content_length = file_size
         if 'Content-Encoding' in file.metadata:
             response.content_encoding = file.metadata['Content-Encoding']
         return response
@@ -522,7 +675,7 @@ class ObjectController(object):
             return Response(status='507 %s is not mounted' % device)
         response_class = HTTPNoContent
         file = DiskFile(self.devices, device, partition, account, container,
-                        obj, disk_chunk_size=self.disk_chunk_size)
+                        obj, self.logger, disk_chunk_size=self.disk_chunk_size)
         if file.is_deleted():
             response_class = HTTPNotFound
         metadata = {
@@ -543,27 +696,31 @@ class ObjectController(object):
         Handle REPLICATE requests for the Swift Object Server.  This is used
         by the object replicator to get hashes for directories.
         """
-        device, partition, suffix = split_path(
-            unquote(request.path), 2, 3, True)
+        try:
+            device, partition, suffix = split_path(
+                unquote(request.path), 2, 3, True)
+        except ValueError, e:
+            return HTTPBadRequest(body=str(e), request=request,
+                                  content_type='text/plain')
         if self.mount_check and not check_mount(self.devices, device):
             return Response(status='507 %s is not mounted' % device)
-        if suffix:
-            recalculate_hashes(os.path.join(self.devices, device,
-                DATADIR, partition), suffix.split('-'))
-            return Response()
         path = os.path.join(self.devices, device, DATADIR, partition)
         if not os.path.exists(path):
             mkdirs(path)
-        _, hashes = get_hashes(path, do_listdir=False)
+        suffixes = suffix.split('-') if suffix else []
+        _junk, hashes = tpool.execute(tpooled_get_hashes, path,
+                                      recalculate=suffixes)
+        # See tpooled_get_hashes "Hack".
+        if isinstance(hashes, BaseException):
+            raise hashes
         return Response(body=pickle.dumps(hashes))
 
     def __call__(self, env, start_response):
         """WSGI Application entry point for the Swift Object Server."""
         start_time = time.time()
         req = Request(env)
-        if req.path_info == '/healthcheck':
-            return healthcheck(req)(env, start_response)
-        elif not check_xml_encodable(req.path_info):
+        self.logger.txn_id = req.headers.get('x-cf-trans-id', None)
+        if not check_utf8(req.path_info):
             res = HTTPPreconditionFailed(body='Invalid UTF8')
         else:
             try:
@@ -571,11 +728,9 @@ class ObjectController(object):
                     res = getattr(self, req.method)(req)
                 else:
                     res = HTTPMethodNotAllowed()
-            except:
-                self.logger.exception('ERROR __call__ error with %s %s '
-                    'transaction %s' % (env.get('REQUEST_METHOD', '-'),
-                    env.get('PATH_INFO', '-'), env.get('HTTP_X_CF_TRANS_ID',
-                    '-')))
+            except Exception:
+                self.logger.exception(_('ERROR __call__ error with %(method)s'
+                    ' %(path)s '), {'method': req.method, 'path': req.path})
                 res = HTTPInternalServerError(body=traceback.format_exc())
         trans_time = time.time() - start_time
         if self.log_requests:
@@ -597,3 +752,10 @@ class ObjectController(object):
             if slow > 0:
                 sleep(slow)
         return res(env, start_response)
+
+
+def app_factory(global_conf, **local_conf):
+    """paste.deploy app factory for creating WSGI object server apps"""
+    conf = global_conf.copy()
+    conf.update(local_conf)
+    return ObjectController(conf)
