@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -14,40 +14,39 @@
 # limitations under the License.
 
 import cPickle as pickle
-import errno
-import logging
 import os
 import signal
 import sys
 import time
 from random import random
 
-from eventlet import patcher, Timeout
+from eventlet import patcher, Timeout, TimeoutError
 
 from swift.common.bufferedhttp import http_connect
 from swift.common.exceptions import ConnectionTimeout
 from swift.common.ring import Ring
-from swift.common.utils import get_logger, renamer
-from swift.common.db_replicator import ReplConnection
+from swift.common.utils import get_logger, renamer, write_pickle
+from swift.common.daemon import Daemon
 from swift.obj.server import ASYNCDIR
 
 
-class ObjectUpdater(object):
+class ObjectUpdater(Daemon):
     """Update object information in container listings."""
 
-    def __init__(self, server_conf, updater_conf):
-        self.logger = get_logger(updater_conf, 'object-updater')
-        self.devices = server_conf.get('devices', '/srv/node')
-        self.mount_check = server_conf.get('mount_check', 'true').lower() in \
+    def __init__(self, conf):
+        self.conf = conf
+        self.logger = get_logger(conf, log_route='object-updater')
+        self.devices = conf.get('devices', '/srv/node')
+        self.mount_check = conf.get('mount_check', 'true').lower() in \
                               ('true', 't', '1', 'on', 'yes', 'y')
-        swift_dir = server_conf.get('swift_dir', '/etc/swift')
-        self.interval = int(updater_conf.get('interval', 300))
+        swift_dir = conf.get('swift_dir', '/etc/swift')
+        self.interval = int(conf.get('interval', 300))
         self.container_ring_path = os.path.join(swift_dir, 'container.ring.gz')
         self.container_ring = None
-        self.concurrency = int(updater_conf.get('concurrency', 1))
-        self.slowdown = float(updater_conf.get('slowdown', 0.01))
-        self.node_timeout = int(updater_conf.get('node_timeout', 10))
-        self.conn_timeout = float(updater_conf.get('conn_timeout', 0.5))
+        self.concurrency = int(conf.get('concurrency', 1))
+        self.slowdown = float(conf.get('slowdown', 0.01))
+        self.node_timeout = int(conf.get('node_timeout', 10))
+        self.conn_timeout = float(conf.get('conn_timeout', 0.5))
         self.successes = 0
         self.failures = 0
 
@@ -55,15 +54,15 @@ class ObjectUpdater(object):
         """Get the container ring.  Load it, if it hasn't been yet."""
         if not self.container_ring:
             self.logger.debug(
-                'Loading container ring from %s' % self.container_ring_path)
+                _('Loading container ring from %s'), self.container_ring_path)
             self.container_ring = Ring(self.container_ring_path)
         return self.container_ring
 
-    def update_forever(self):   # pragma: no cover
+    def run_forever(self, *args, **kwargs):
         """Run the updater continuously."""
         time.sleep(random() * self.interval)
         while True:
-            self.logger.info('Begin object update sweep')
+            self.logger.info(_('Begin object update sweep'))
             begin = time.time()
             pids = []
             # read from container ring to ensure it's fresh
@@ -72,7 +71,7 @@ class ObjectUpdater(object):
                 if self.mount_check and not \
                         os.path.ismount(os.path.join(self.devices, device)):
                     self.logger.warn(
-                        'Skipping %s as it is not mounted' % device)
+                        _('Skipping %s as it is not mounted'), device)
                     continue
                 while len(pids) >= self.concurrency:
                     pids.remove(os.wait()[0])
@@ -87,20 +86,23 @@ class ObjectUpdater(object):
                     forkbegin = time.time()
                     self.object_sweep(os.path.join(self.devices, device))
                     elapsed = time.time() - forkbegin
-                    self.logger.info('Object update sweep of %s completed: '
-                        '%.02fs, %s successes, %s failures' %
-                        (device, elapsed, self.successes, self.failures))
+                    self.logger.info(_('Object update sweep of %(device)s'
+                        ' completed: %(elapsed).02fs, %(success)s successes'
+                        ', %(fail)s failures'),
+                        {'device': device, 'elapsed': elapsed,
+                         'success': self.successes, 'fail': self.failures})
                     sys.exit()
             while pids:
                 pids.remove(os.wait()[0])
             elapsed = time.time() - begin
-            self.logger.info('Object update sweep completed: %.02fs' % elapsed)
+            self.logger.info(_('Object update sweep completed: %.02fs'),
+                    elapsed)
             if elapsed < self.interval:
                 time.sleep(self.interval - elapsed)
 
-    def update_once_single_threaded(self):
+    def run_once(self, *args, **kwargs):
         """Run the updater once"""
-        self.logger.info('Begin object update single threaded sweep')
+        self.logger.info(_('Begin object update single threaded sweep'))
         begin = time.time()
         self.successes = 0
         self.failures = 0
@@ -108,13 +110,14 @@ class ObjectUpdater(object):
             if self.mount_check and \
                     not os.path.ismount(os.path.join(self.devices, device)):
                 self.logger.warn(
-                    'Skipping %s as it is not mounted' % device)
+                    _('Skipping %s as it is not mounted'), device)
                 continue
             self.object_sweep(os.path.join(self.devices, device))
         elapsed = time.time() - begin
-        self.logger.info('Object update single threaded sweep completed: '
-            '%.02fs, %s successes, %s failures' %
-            (elapsed, self.successes, self.failures))
+        self.logger.info(_('Object update single threaded sweep completed: '
+            '%(elapsed).02fs, %(success)s successes, %(fail)s failures'),
+            {'elapsed': elapsed, 'success': self.successes,
+             'fail': self.failures})
 
     def object_sweep(self, device):
         """
@@ -149,29 +152,37 @@ class ObjectUpdater(object):
         """
         try:
             update = pickle.load(open(update_path, 'rb'))
-        except Exception, err:
+        except Exception:
             self.logger.exception(
-                'ERROR Pickle problem, quarantining %s' % update_path)
+                _('ERROR Pickle problem, quarantining %s'), update_path)
             renamer(update_path, os.path.join(device,
                 'quarantined', 'objects', os.path.basename(update_path)))
             return
+        successes = update.get('successes', [])
         part, nodes = self.get_container_ring().get_nodes(
                                 update['account'], update['container'])
         obj = '/%s/%s/%s' % \
               (update['account'], update['container'], update['obj'])
         success = True
         for node in nodes:
-            status = self.object_update(node, part, update['op'], obj,
+            if node['id'] not in successes:
+                status = self.object_update(node, part, update['op'], obj,
                                         update['headers'])
-            if not (200 <= status < 300) and status != 404:
-                success = False
+                if not (200 <= status < 300) and status != 404:
+                    success = False
+                else:
+                    successes.append(node['id'])
         if success:
             self.successes += 1
-            self.logger.debug('Update sent for %s %s' % (obj, update_path))
+            self.logger.debug(_('Update sent for %(obj)s %(path)s'),
+                {'obj': obj, 'path': update_path})
             os.unlink(update_path)
         else:
             self.failures += 1
-            self.logger.debug('Update failed for %s %s' % (obj, update_path))
+            self.logger.debug(_('Update failed for %(obj)s %(path)s'),
+                {'obj': obj, 'path': update_path})
+            update['successes'] = successes
+            write_pickle(update, update_path, os.path.join(device, 'tmp'))
 
     def object_update(self, node, part, op, obj, headers):
         """
@@ -191,7 +202,7 @@ class ObjectUpdater(object):
                 resp = conn.getresponse()
                 resp.read()
                 return resp.status
-        except:
-            self.logger.exception('ERROR with remote server '
-                '%(ip)s:%(port)s/%(device)s' % node)
+        except (Exception, TimeoutError):
+            self.logger.exception(_('ERROR with remote server '
+                '%(ip)s:%(port)s/%(device)s'), node)
         return 500
