@@ -1,5 +1,5 @@
 #!/usr/bin/python -u
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -15,25 +15,27 @@
 # limitations under the License.
 
 import array
-import configobj
 from datetime import datetime
 import locale
 import os
+import os.path
 import random
 import StringIO
+import sys
 import time
 import threading
 import uuid
 import unittest
 import urllib
 
+from test import get_config
 from swift import Account, AuthenticationFailed, Connection, Container, \
      File, ResponseError
 
-config = configobj.ConfigObj(os.environ['SWIFT_TEST_CONFIG_FILE'])
+config = get_config()
+
 locale.setlocale(locale.LC_COLLATE, config.get('collate', 'C'))
 
-NoRun = object
 
 class Base:
     pass
@@ -106,9 +108,12 @@ class Base(unittest.TestCase):
         self.assert_(response_body == body,
            'Body returned: %s' % (response_body))
 
-    def assert_status(self, status):
-        self.assert_(self.env.conn.response.status == status,
-           'Status returned: %d' % (self.env.conn.response.status))
+    def assert_status(self, status_or_statuses):
+        self.assert_(self.env.conn.response.status == status_or_statuses or
+                     (hasattr(status_or_statuses, '__iter__') and
+                      self.env.conn.response.status in status_or_statuses),
+           'Status returned: %d Expected: %s' %
+           (self.env.conn.response.status, status_or_statuses))
 
 class Base2(object):
     def setUp(self):
@@ -123,7 +128,8 @@ class TestAccountEnv:
     def setUp(cls):
         cls.conn = Connection(config)
         cls.conn.authenticate()
-        cls.account = Account(cls.conn, config['account'])
+        cls.account = Account(cls.conn, config.get('account',
+                                                   config['username']))
         cls.account.delete_containers()
 
         cls.containers = []
@@ -148,11 +154,11 @@ class TestAccount(Base):
     def testNoAuthToken(self):
         self.assertRaises(ResponseError, self.env.account.info,
             cfg={'no_auth_token':True})
-        self.assert_status(412)
+        self.assert_status([401, 412])
 
         self.assertRaises(ResponseError, self.env.account.containers,
             cfg={'no_auth_token':True})
-        self.assert_status(412)
+        self.assert_status([401, 412])
 
     def testInvalidUTF8Path(self):
         invalid_utf8 = Utils.create_utf8_name()[::-1]
@@ -167,9 +173,18 @@ class TestAccount(Base):
         self.assert_status(412)
         self.assert_body('Bad URL')
 
+    def testInvalidPath(self):
+        was_url = self.env.account.conn.storage_url
+        self.env.account.conn.storage_url = "/%s" % was_url
+        self.env.account.conn.make_request('GET')
+        try:
+            self.assert_status(404)
+        finally:
+            self.env.account.conn.storage_url = was_url
+
     def testPUT(self):
         self.env.account.conn.make_request('PUT')
-        self.assert_status(405)
+        self.assert_status([403, 405])
 
     def testAccountHead(self):
         try_count = 0
@@ -292,7 +307,8 @@ class TestAccountNoContainersEnv:
     def setUp(cls):
         cls.conn = Connection(config)
         cls.conn.authenticate()
-        cls.account = Account(cls.conn, config['account'])
+        cls.account = Account(cls.conn, config.get('account',
+                                                   config['username']))
         cls.account.delete_containers()
 
 class TestAccountNoContainers(Base):
@@ -317,7 +333,8 @@ class TestContainerEnv:
     def setUp(cls):
         cls.conn = Connection(config)
         cls.conn.authenticate()
-        cls.account = Account(cls.conn, config['account'])
+        cls.account = Account(cls.conn, config.get('account',
+                                                   config['username']))
         cls.account.delete_containers()
 
         cls.container = cls.account.container(Utils.create_name())
@@ -582,15 +599,6 @@ class TestContainer(Base):
             (cont.name))
         self.assert_status(400)
 
-    def testYourmomContainer(self):
-        cont = self.env.account.container('yourmom')
-        self.assert_(cont.create())
-
-        info = cont.info()
-        self.assert_status(204)
-        self.assertEquals(info['object_count'], 0)
-        self.assertEquals(info['bytes_used'], 0)
-
     def testContainerExistenceCachingProblem(self):
         cont = self.env.account.container(Utils.create_name())
         self.assertRaises(ResponseError, cont.files)
@@ -611,7 +619,8 @@ class TestContainerPathsEnv:
     def setUp(cls):
         cls.conn = Connection(config)
         cls.conn.authenticate()
-        cls.account = Account(cls.conn, config['account'])
+        cls.account = Account(cls.conn, config.get('account',
+                                                   config['username']))
         cls.account.delete_containers()
 
         cls.file_size = 8
@@ -771,7 +780,8 @@ class TestFileEnv:
     def setUp(cls):
         cls.conn = Connection(config)
         cls.conn.authenticate()
-        cls.account = Account(cls.conn, config['account'])
+        cls.account = Account(cls.conn, config.get('account',
+                                                   config['username']))
         cls.account.delete_containers()
 
         cls.container = cls.account.container(Utils.create_name())
@@ -1132,7 +1142,8 @@ class TestFile(Base):
         self.assert_status(400)
 
         # bad request types
-        for req in ('LICK', 'GETorHEAD_base', 'container_info', 'best_response'):
+        #for req in ('LICK', 'GETorHEAD_base', 'container_info', 'best_response'):
+        for req in ('LICK', 'GETorHEAD_base'):
             self.env.account.conn.make_request(req)
             self.assert_status(405)
 
@@ -1416,7 +1427,8 @@ class TestFileComparisonEnv:
     def setUp(cls):
         cls.conn = Connection(config)
         cls.conn.authenticate()
-        cls.account = Account(cls.conn, config['account'])
+        cls.account = Account(cls.conn, config.get('account',
+                                                   config['username']))
         cls.account.delete_containers()
 
         cls.container = cls.account.container(Utils.create_name())
