@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -13,21 +13,34 @@
 # See the License for the specific language governing permissions and
 # limitations under the License.
 
-from os import kill
+from os import environ, kill
 from signal import SIGTERM
 from subprocess import call, Popen
 from time import sleep
+from ConfigParser import ConfigParser
 
 from swift.common.bufferedhttp import http_connect_raw as http_connect
 from swift.common.client import get_auth
 from swift.common.ring import Ring
 
 
+SUPER_ADMIN_KEY = None
+
+c = ConfigParser()
+PROXY_SERVER_CONF_FILE = environ.get('SWIFT_PROXY_SERVER_CONF_FILE',
+                                     '/etc/swift/proxy-server.conf')
+if c.read(PROXY_SERVER_CONF_FILE):
+    conf = dict(c.items('filter:swauth'))
+    SUPER_ADMIN_KEY = conf.get('super_admin_key', 'swauthkey')
+else:
+    exit('Unable to read config file: %s' % PROXY_SERVER_CONF_FILE)
+
+
 def kill_pids(pids):
     for pid in pids.values():
         try:
             kill(pid, SIGTERM)
-        except:
+        except Exception:
             pass
 
 
@@ -35,8 +48,6 @@ def reset_environment():
     call(['resetswift'])
     pids = {}
     try:
-        pids['auth'] = Popen(['swift-auth-server',
-                              '/etc/swift/auth-server.conf']).pid
         pids['proxy'] = Popen(['swift-proxy-server',
                                '/etc/swift/proxy-server.conf']).pid
         port2server = {}
@@ -50,12 +61,9 @@ def reset_environment():
         container_ring = Ring('/etc/swift/container.ring.gz')
         object_ring = Ring('/etc/swift/object.ring.gz')
         sleep(5)
-        conn = http_connect('127.0.0.1', '11000', 'POST', '/recreate_accounts')
-        resp = conn.getresponse()
-        if resp.status != 200:
-            raise Exception('Recreating accounts failed. (%d)' % resp.status)
-        url, token = \
-            get_auth('http://127.0.0.1:11000/auth', 'test:tester', 'testing')
+        call(['recreateaccounts'])
+        url, token = get_auth('http://127.0.0.1:8080/auth/v1.0',
+                              'test:tester', 'testing')
         account = url.split('/')[-1]
     except BaseException, err:
         kill_pids(pids)
