@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -196,6 +196,105 @@ class TestAccountController(unittest.TestCase):
         self.assertEquals(resp.status_int, 403)
         self.assertEquals(resp.body, 'Recently deleted')
 
+    def test_PUT_GET_metadata(self):
+        # Set metadata header
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': normalize_timestamp(1),
+                     'X-Account-Meta-Test': 'Value'})
+        resp = self.controller.PUT(req)
+        self.assertEquals(resp.status_int, 201)
+        req = Request.blank('/sda1/p/a')
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-account-meta-test'), 'Value')
+        # Set another metadata header, ensuring old one doesn't disappear
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': normalize_timestamp(1),
+                     'X-Account-Meta-Test2': 'Value2'})
+        resp = self.controller.POST(req)
+        self.assertEquals(resp.status_int, 204)
+        req = Request.blank('/sda1/p/a')
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-account-meta-test'), 'Value')
+        self.assertEquals(resp.headers.get('x-account-meta-test2'), 'Value2')
+        # Update metadata header
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': normalize_timestamp(3),
+                     'X-Account-Meta-Test': 'New Value'})
+        resp = self.controller.PUT(req)
+        self.assertEquals(resp.status_int, 202)
+        req = Request.blank('/sda1/p/a')
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-account-meta-test'), 'New Value')
+        # Send old update to metadata header
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': normalize_timestamp(2),
+                     'X-Account-Meta-Test': 'Old Value'})
+        resp = self.controller.PUT(req)
+        self.assertEquals(resp.status_int, 202)
+        req = Request.blank('/sda1/p/a')
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-account-meta-test'), 'New Value')
+        # Remove metadata header (by setting it to empty)
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': normalize_timestamp(4),
+                     'X-Account-Meta-Test': ''})
+        resp = self.controller.PUT(req)
+        self.assertEquals(resp.status_int, 202)
+        req = Request.blank('/sda1/p/a')
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assert_('x-account-meta-test' not in resp.headers)
+
+    def test_POST_HEAD_metadata(self):
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': normalize_timestamp(1)})
+        resp = self.controller.PUT(req)
+        self.assertEquals(resp.status_int, 201)
+        # Set metadata header
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': normalize_timestamp(1),
+                     'X-Account-Meta-Test': 'Value'})
+        resp = self.controller.POST(req)
+        self.assertEquals(resp.status_int, 204)
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'HEAD'})
+        resp = self.controller.HEAD(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-account-meta-test'), 'Value')
+        # Update metadata header
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': normalize_timestamp(3),
+                     'X-Account-Meta-Test': 'New Value'})
+        resp = self.controller.POST(req)
+        self.assertEquals(resp.status_int, 204)
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'HEAD'})
+        resp = self.controller.HEAD(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-account-meta-test'), 'New Value')
+        # Send old update to metadata header
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': normalize_timestamp(2),
+                     'X-Account-Meta-Test': 'Old Value'})
+        resp = self.controller.POST(req)
+        self.assertEquals(resp.status_int, 204)
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'HEAD'})
+        resp = self.controller.HEAD(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-account-meta-test'), 'New Value')
+        # Remove metadata header (by setting it to empty)
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': normalize_timestamp(4),
+                     'X-Account-Meta-Test': ''})
+        resp = self.controller.POST(req)
+        self.assertEquals(resp.status_int, 204)
+        req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'HEAD'})
+        resp = self.controller.HEAD(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assert_('x-account-meta-test' not in resp.headers)
+
     def test_GET_not_found_plain(self):
         req = Request.blank('/sda1/p/a', environ={'REQUEST_METHOD': 'GET'})
         resp = self.controller.GET(req)
@@ -795,33 +894,6 @@ class TestAccountController(unittest.TestCase):
                         listing.append(node2.firstChild.nodeValue)
         self.assertEquals(listing, ['sub.1.0', 'sub.1.1', 'sub.1.2'])
 
-    def test_healthcheck(self):
-        inbuf = StringIO()
-        errbuf = StringIO()
-        outbuf = StringIO()
-
-        def start_response(*args):
-            """ Sends args to outbuf """
-            outbuf.writelines(args)
-
-        self.controller.__call__({'REQUEST_METHOD': 'GET',
-                                         'SCRIPT_NAME': '',
-                                         'PATH_INFO': '/healthcheck',
-                                         'SERVER_NAME': '127.0.0.1',
-                                         'SERVER_PORT': '8080',
-                                         'SERVER_PROTOCOL': 'HTTP/1.0',
-                                         'CONTENT_LENGTH': '0',
-                                         'wsgi.version': (1, 0),
-                                         'wsgi.url_scheme': 'http',
-                                         'wsgi.input': inbuf,
-                                         'wsgi.errors': errbuf,
-                                         'wsgi.multithread': False,
-                                         'wsgi.multiprocess': False,
-                                         'wsgi.run_once': False},
-                                        start_response)
-        self.assertEquals(errbuf.getvalue(), '')
-        self.assertEquals(outbuf.getvalue()[:4], '200 ')
-
     def test_through_call(self):
         inbuf = StringIO()
         errbuf = StringIO()
