@@ -1,4 +1,4 @@
-# Copyright (c) 2010 OpenStack, LLC.
+# Copyright (c) 2010-2011 OpenStack, LLC.
 #
 # Licensed under the Apache License, Version 2.0 (the "License");
 # you may not use this file except in compliance with the License.
@@ -19,6 +19,7 @@ import unittest
 from shutil import rmtree
 from StringIO import StringIO
 from time import time
+from tempfile import mkdtemp
 
 from eventlet import spawn, TimeoutError, listen
 from eventlet.timeout import Timeout
@@ -33,17 +34,8 @@ class TestContainerController(unittest.TestCase):
     """ Test swift.container_server.ContainerController """
     def setUp(self):
         """ Set up for testing swift.object_server.ObjectController """
-        self.path_to_test_xfs = os.environ.get('PATH_TO_TEST_XFS')
-        if not self.path_to_test_xfs or \
-                not os.path.exists(self.path_to_test_xfs):
-            print >>sys.stderr, 'WARNING: PATH_TO_TEST_XFS not set or not ' \
-                'pointing to a valid directory.\n' \
-                'Please set PATH_TO_TEST_XFS to a directory on an XFS file ' \
-                'system for testing.'
-            self.testdir = '/tmp/SWIFTUNITTEST'
-        else:
-            self.testdir = os.path.join(self.path_to_test_xfs,
-                            'tmp_test_object_server_ObjectController')
+        self.testdir = os.path.join(mkdtemp(),
+                                    'tmp_test_object_server_ObjectController')
         mkdirs(self.testdir)
         rmtree(self.testdir)
         mkdirs(os.path.join(self.testdir, 'sda1'))
@@ -53,7 +45,50 @@ class TestContainerController(unittest.TestCase):
 
     def tearDown(self):
         """ Tear down for testing swift.object_server.ObjectController """
-        rmtree(self.testdir, ignore_errors=1)
+        rmtree(os.path.dirname(self.testdir), ignore_errors=1)
+
+    def test_acl_container(self):
+        # Ensure no acl by default
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': '0'})
+        self.controller.PUT(req)
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'HEAD'})
+        response = self.controller.HEAD(req)
+        self.assert_(response.status.startswith('204'))
+        self.assert_('x-container-read' not in response.headers)
+        self.assert_('x-container-write' not in response.headers)
+        # Ensure POSTing acls works
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': '1', 'X-Container-Read': '.r:*',
+                     'X-Container-Write': 'account:user'})
+        self.controller.POST(req)
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'HEAD'})
+        response = self.controller.HEAD(req)
+        self.assert_(response.status.startswith('204'))
+        self.assertEquals(response.headers.get('x-container-read'), '.r:*')
+        self.assertEquals(response.headers.get('x-container-write'),
+                          'account:user')
+        # Ensure we can clear acls on POST
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': '3', 'X-Container-Read': '',
+                     'X-Container-Write': ''})
+        self.controller.POST(req)
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'HEAD'})
+        response = self.controller.HEAD(req)
+        self.assert_(response.status.startswith('204'))
+        self.assert_('x-container-read' not in response.headers)
+        self.assert_('x-container-write' not in response.headers)
+        # Ensure PUTing acls works
+        req = Request.blank('/sda1/p/a/c2', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': '4', 'X-Container-Read': '.r:*',
+                     'X-Container-Write': 'account:user'})
+        self.controller.PUT(req)
+        req = Request.blank('/sda1/p/a/c2', environ={'REQUEST_METHOD': 'HEAD'})
+        response = self.controller.HEAD(req)
+        self.assert_(response.status.startswith('204'))
+        self.assertEquals(response.headers.get('x-container-read'), '.r:*')
+        self.assertEquals(response.headers.get('x-container-write'),
+                          'account:user')
 
     def test_HEAD(self):
         req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'PUT',
@@ -93,6 +128,109 @@ class TestContainerController(unittest.TestCase):
         resp = self.controller.PUT(req)
         self.assertEquals(resp.status_int, 404)
 
+    def test_PUT_GET_metadata(self):
+        # Set metadata header
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': normalize_timestamp(1),
+                     'X-Container-Meta-Test': 'Value'})
+        resp = self.controller.PUT(req)
+        self.assertEquals(resp.status_int, 201)
+        req = Request.blank('/sda1/p/a/c')
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-container-meta-test'), 'Value')
+        # Set another metadata header, ensuring old one doesn't disappear
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': normalize_timestamp(1),
+                     'X-Container-Meta-Test2': 'Value2'})
+        resp = self.controller.POST(req)
+        self.assertEquals(resp.status_int, 204)
+        req = Request.blank('/sda1/p/a/c')
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-container-meta-test'), 'Value')
+        self.assertEquals(resp.headers.get('x-container-meta-test2'), 'Value2')
+        # Update metadata header
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': normalize_timestamp(3),
+                     'X-Container-Meta-Test': 'New Value'})
+        resp = self.controller.PUT(req)
+        self.assertEquals(resp.status_int, 202)
+        req = Request.blank('/sda1/p/a/c')
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-container-meta-test'),
+                          'New Value')
+        # Send old update to metadata header
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': normalize_timestamp(2),
+                     'X-Container-Meta-Test': 'Old Value'})
+        resp = self.controller.PUT(req)
+        self.assertEquals(resp.status_int, 202)
+        req = Request.blank('/sda1/p/a/c')
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-container-meta-test'),
+                          'New Value')
+        # Remove metadata header (by setting it to empty)
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': normalize_timestamp(4),
+                     'X-Container-Meta-Test': ''})
+        resp = self.controller.PUT(req)
+        self.assertEquals(resp.status_int, 202)
+        req = Request.blank('/sda1/p/a/c')
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assert_('x-container-meta-test' not in resp.headers)
+
+    def test_POST_HEAD_metadata(self):
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'PUT'},
+            headers={'X-Timestamp': normalize_timestamp(1)})
+        resp = self.controller.PUT(req)
+        self.assertEquals(resp.status_int, 201)
+        # Set metadata header
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': normalize_timestamp(1),
+                     'X-Container-Meta-Test': 'Value'})
+        resp = self.controller.POST(req)
+        self.assertEquals(resp.status_int, 204)
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'HEAD'})
+        resp = self.controller.HEAD(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-container-meta-test'), 'Value')
+        # Update metadata header
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': normalize_timestamp(3),
+                     'X-Container-Meta-Test': 'New Value'})
+        resp = self.controller.POST(req)
+        self.assertEquals(resp.status_int, 204)
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'HEAD'})
+        resp = self.controller.HEAD(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-container-meta-test'),
+                          'New Value')
+        # Send old update to metadata header
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': normalize_timestamp(2),
+                     'X-Container-Meta-Test': 'Old Value'})
+        resp = self.controller.POST(req)
+        self.assertEquals(resp.status_int, 204)
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'HEAD'})
+        resp = self.controller.HEAD(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.headers.get('x-container-meta-test'),
+                          'New Value')
+        # Remove metadata header (by setting it to empty)
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'POST'},
+            headers={'X-Timestamp': normalize_timestamp(4),
+                     'X-Container-Meta-Test': ''})
+        resp = self.controller.POST(req)
+        self.assertEquals(resp.status_int, 204)
+        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'HEAD'})
+        resp = self.controller.HEAD(req)
+        self.assertEquals(resp.status_int, 204)
+        self.assert_('x-container-meta-test' not in resp.headers)
+
     def test_DELETE_obj_not_found(self):
         req = Request.blank('/sda1/p/a/c/o',
                 environ={'REQUEST_METHOD': 'DELETE'},
@@ -333,18 +471,20 @@ class TestContainerController(unittest.TestCase):
         resp = self.controller.GET(req)
         self.assertEquals(resp.status_int, 412)
 
-    def test_GET_format(self):
+    def test_GET_json(self):
         # make a container
-        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'PUT',
+        req = Request.blank('/sda1/p/a/jsonc', environ={'REQUEST_METHOD': 'PUT',
             'HTTP_X_TIMESTAMP': '0'})
         resp = self.controller.PUT(req)
         # test an empty container
-        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'GET'})
+        req = Request.blank('/sda1/p/a/jsonc?format=json',
+            environ={'REQUEST_METHOD': 'GET'})
         resp = self.controller.GET(req)
-        self.assertEquals(resp.status_int, 204)
+        self.assertEquals(resp.status_int, 200)
+        self.assertEquals(eval(resp.body), [])
         # fill the container
         for i in range(3):
-            req = Request.blank('/sda1/p/a/c/%s'%i, environ=
+            req = Request.blank('/sda1/p/a/jsonc/%s'%i, environ=
                     {'REQUEST_METHOD': 'PUT',
                     'HTTP_X_TIMESTAMP': '1',
                     'HTTP_X_CONTENT_TYPE': 'text/plain',
@@ -368,8 +508,88 @@ class TestContainerController(unittest.TestCase):
                     "bytes":0,
                     "content_type":"text/plain",
                     "last_modified":"1970-01-01T00:00:01"}]
+
+        req = Request.blank('/sda1/p/a/jsonc?format=json',
+                environ={'REQUEST_METHOD': 'GET'})
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.content_type, 'application/json')
+        self.assertEquals(eval(resp.body), json_body)
+
+        for accept in ('application/json', 'application/json;q=1.0,*/*;q=0.9',
+                 '*/*;q=0.9,application/json;q=1.0', 'application/*'):
+            req = Request.blank('/sda1/p/a/jsonc',
+                    environ={'REQUEST_METHOD': 'GET'})
+            req.accept = accept
+            resp = self.controller.GET(req)
+            self.assertEquals(eval(resp.body), json_body,
+                'Invalid body for Accept: %s' % accept)
+            self.assertEquals(resp.content_type, 'application/json',
+                'Invalid content_type for Accept: %s' % accept)
+
+    def test_GET_plain(self):
+        # make a container
+        req = Request.blank('/sda1/p/a/plainc', environ={'REQUEST_METHOD': 'PUT',
+            'HTTP_X_TIMESTAMP': '0'})
+        resp = self.controller.PUT(req)
+        # test an empty container
+        req = Request.blank('/sda1/p/a/plainc', environ={'REQUEST_METHOD': 'GET'})
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.status_int, 204)
+        # fill the container
+        for i in range(3):
+            req = Request.blank('/sda1/p/a/plainc/%s'%i, environ=
+                    {'REQUEST_METHOD': 'PUT',
+                    'HTTP_X_TIMESTAMP': '1',
+                    'HTTP_X_CONTENT_TYPE': 'text/plain',
+                    'HTTP_X_ETAG': 'x',
+                    'HTTP_X_SIZE': 0})
+            resp = self.controller.PUT(req)
+            self.assertEquals(resp.status_int, 201)
+        plain_body = '0\n1\n2\n'
+
+        req = Request.blank('/sda1/p/a/plainc',
+                environ={'REQUEST_METHOD': 'GET'})
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.content_type, 'text/plain')
+        self.assertEquals(resp.body, plain_body)
+
+        for accept in ('', 'text/plain', 'application/xml;q=0.8,*/*;q=0.9',
+                '*/*;q=0.9,application/xml;q=0.8', '*/*',
+                'text/plain,application/xml'):
+            req = Request.blank('/sda1/p/a/plainc',
+                    environ={'REQUEST_METHOD': 'GET'})
+            req.accept = accept
+            resp = self.controller.GET(req)
+            self.assertEquals(resp.body, plain_body,
+                'Invalid body for Accept: %s' % accept)
+            self.assertEquals(resp.content_type, 'text/plain',
+                'Invalid content_type for Accept: %s' % accept)
+
+        # test conflicting formats
+        req = Request.blank('/sda1/p/a/plainc?format=plain',
+                environ={'REQUEST_METHOD': 'GET'})
+        req.accept = 'application/json'
+        resp = self.controller.GET(req)
+        self.assertEquals(resp.content_type, 'text/plain')
+        self.assertEquals(resp.body, plain_body)
+
+    def test_GET_xml(self):
+        # make a container
+        req = Request.blank('/sda1/p/a/xmlc', environ={'REQUEST_METHOD': 'PUT',
+            'HTTP_X_TIMESTAMP': '0'})
+        resp = self.controller.PUT(req)
+        # fill the container
+        for i in range(3):
+            req = Request.blank('/sda1/p/a/xmlc/%s'%i, environ=
+                    {'REQUEST_METHOD': 'PUT',
+                    'HTTP_X_TIMESTAMP': '1',
+                    'HTTP_X_CONTENT_TYPE': 'text/plain',
+                    'HTTP_X_ETAG': 'x',
+                    'HTTP_X_SIZE': 0})
+            resp = self.controller.PUT(req)
+            self.assertEquals(resp.status_int, 201)
         xml_body = '<?xml version="1.0" encoding="UTF-8"?>\n' \
-            '<container name="c">' \
+            '<container name="xmlc">' \
                 '<object><name>0</name><hash>x</hash><bytes>0</bytes>' \
                     '<content_type>text/plain</content_type>' \
                     '<last_modified>1970-01-01T00:00:01' \
@@ -383,46 +603,30 @@ class TestContainerController(unittest.TestCase):
                     '<last_modified>1970-01-01T00:00:01' \
                     '</last_modified></object>' \
             '</container>'
-        plain_body = '0\n1\n2\n'
-        req = Request.blank('/sda1/p/a/c?format=json', environ={'REQUEST_METHOD': 'GET'})
-        resp = self.controller.GET(req)
-        self.assertEquals(resp.content_type, 'application/json')
-        result = eval(resp.body)
-        self.assertEquals(result, json_body)
-        req = Request.blank('/sda1/p/a/c?format=xml', environ={'REQUEST_METHOD': 'GET'})
+        # tests
+        req = Request.blank('/sda1/p/a/xmlc?format=xml',
+                environ={'REQUEST_METHOD': 'GET'})
         resp = self.controller.GET(req)
         self.assertEquals(resp.content_type, 'application/xml')
-        result = resp.body
-        self.assertEquals(result, xml_body)
-        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'GET'})
-        req.accept = 'application/json'
+        self.assertEquals(resp.body, xml_body)
+
+        for xml_accept in ('application/xml', 'application/xml;q=1.0,*/*;q=0.9',
+                 '*/*;q=0.9,application/xml;q=1.0', 'application/xml,text/xml'):
+            req = Request.blank('/sda1/p/a/xmlc',
+                    environ={'REQUEST_METHOD': 'GET'})
+            req.accept = xml_accept
+            resp = self.controller.GET(req)
+            self.assertEquals(resp.body, xml_body,
+                'Invalid body for Accept: %s' % xml_accept)
+            self.assertEquals(resp.content_type, 'application/xml',
+                'Invalid content_type for Accept: %s' % xml_accept)
+
+        req = Request.blank('/sda1/p/a/xmlc',
+                environ={'REQUEST_METHOD': 'GET'})
+        req.accept = 'text/xml'
         resp = self.controller.GET(req)
-        self.assertEquals(resp.content_type, 'application/json')
-        result = eval(resp.body)
-        self.assertEquals(result, json_body)
-        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'GET'})
-        req.accept = '*/*'
-        resp = self.controller.GET(req)
-        self.assertEquals(resp.content_type, 'text/plain')
-        result = resp.body
-        self.assertEquals(result, plain_body)
-        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'GET'})
-        req.accept = 'application/*'
-        resp = self.controller.GET(req)
-        result = eval(resp.body)
-        self.assertEquals(result, json_body)
-        req = Request.blank('/sda1/p/a/c', environ={'REQUEST_METHOD': 'GET'})
-        req.accept = 'application/xml'
-        resp = self.controller.GET(req)
-        result = resp.body
-        self.assertEquals(result, xml_body)
-        # test conflicting formats
-        req = Request.blank('/sda1/p/a/c?format=plain', environ={'REQUEST_METHOD': 'GET'})
-        req.accept = 'application/json'
-        resp = self.controller.GET(req)
-        self.assertEquals(resp.content_type, 'text/plain')
-        result = resp.body
-        self.assertEquals(result, plain_body)
+        self.assertEquals(resp.content_type, 'text/xml')
+        self.assertEquals(resp.body, xml_body)
 
     def test_GET_marker(self):
         # make a container
@@ -551,33 +755,6 @@ class TestContainerController(unittest.TestCase):
              {"name":"US/TX","hash":"x","bytes":0,"content_type":"text/plain",
               "last_modified":"1970-01-01T00:00:01"}])
 
-    def test_healthcheck(self):
-        inbuf = StringIO()
-        errbuf = StringIO()
-        outbuf = StringIO()
-
-        def start_response(*args):
-            """ Sends args to outbuf """
-            outbuf.writelines(args)
-
-        self.controller.__call__({'REQUEST_METHOD': 'GET',
-                                         'SCRIPT_NAME': '',
-                                         'PATH_INFO': '/healthcheck',
-                                         'SERVER_NAME': '127.0.0.1',
-                                         'SERVER_PORT': '8080',
-                                         'SERVER_PROTOCOL': 'HTTP/1.0',
-                                         'CONTENT_LENGTH': '0',
-                                         'wsgi.version': (1, 0),
-                                         'wsgi.url_scheme': 'http',
-                                         'wsgi.input': inbuf,
-                                         'wsgi.errors': errbuf,
-                                         'wsgi.multithread': False,
-                                         'wsgi.multiprocess': False,
-                                         'wsgi.run_once': False},
-                                        start_response)
-        self.assertEquals(errbuf.getvalue(), '')
-        self.assertEquals(outbuf.getvalue()[:4], '200 ')
-
     def test_through_call(self):
         inbuf = StringIO()
         errbuf = StringIO()
